// ElecDrawDoc.cpp : implementation of the CElecDrawDoc class
//

#include "stdafx.h"
//#include "DlgSplash.h"
#include "ElecDraw.h"
#include "DlgMacroBar.h"
#include "Primitive.h"
//#include "AuxPrim.h"
#include "ViewToolbar.h"
#include "ElecDrawDoc.h"
#include "GridDialog.h"
#include "ElecDrawView.h"
#include "ConfigDialog.h"
#include "LayerComboBox.h"
#include "Mainfrm.h"
#include "DlgSaveBmp.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CElecDrawDoc

IMPLEMENT_DYNCREATE(CElecDrawDoc, CDocument)

BEGIN_MESSAGE_MAP(CElecDrawDoc, CDocument)
	//{{AFX_MSG_MAP(CElecDrawDoc)
	ON_COMMAND(ID_FILE_SAVE, OnFileSave)
	ON_COMMAND(ID_FILE_SAVE_AS, OnFileSaveAs)
	ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
	ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO, OnUpdateEditUndo)
	ON_COMMAND(ID_EDIT_REDO, OnEditRedo)
	ON_UPDATE_COMMAND_UI(ID_EDIT_REDO, OnUpdateEditRedo)
	ON_COMMAND(ID_FILE_SAVEBMP, OnFileSavebmp)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CElecDrawDoc construction/destruction

CElecDrawDoc::CElecDrawDoc()
{
	CMainFrame *wnd = (CMainFrame *)AfxGetMainWnd();
	if( wnd != NULL)
		wnd->m_DocCache.AddTail( (CObject*)this);
}

CElecDrawDoc::~CElecDrawDoc()
{
	CMainFrame *wnd = (CMainFrame *)AfxGetMainWnd();
	if( wnd != NULL)
	{
		POSITION p = wnd->m_DocCache.Find( (CObject*)this);
		wnd->m_DocCache.RemoveAt( p);
	}
}

UINT CElecDrawDoc::GetID()
{
	CDocTemplate* tmpl = GetDocTemplate();
	CString ext;
	tmpl->GetDocString( ext, CDocTemplate::filterExt);
	ext.MakeUpper();

	if( ext == ".FCD")
		return IDR_CHILDFRAME;
	else if( ext == ".FCM")
		return IDR_CHILDFRAMEMACRO;
	else if( ext == ".FCL")
		return IDR_CHILDFRAMELIB;
	else
		return 0;
}

void CElecDrawDoc::DeleteContents() 
{
	int a, n = m_compArray.GetSize();
	for( a=0; a<n; a++)
		delete m_compArray[a];
	m_compArray.RemoveAll();
	m_selArray.RemoveAll();

	if( m_TitleToolBar.m_EditTL.m_hWnd != NULL)
		m_TitleToolBar.m_EditTL.SetWindowText("");

	// Svuota l'undo buffer
	UndoReset();

	CDocument::DeleteContents();
}

/////////////////////////////////////////////////////////////////////////////
// CElecDrawDoc serialization

void CElecDrawDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// Per prima cosa verifica (se questa e' una macro) che il nome non sia
		// gia' utilizzato dalla libreria standard
		if( GetID() == IDR_CHILDFRAMEMACRO)
		{
			CFile *file = ar.GetFile();
			CString titolo( LPCSTR(file->GetFileTitle()));
			titolo.MakeLower();
			int idx = titolo.ReverseFind('.');
			if( idx != -1)
				titolo = titolo.Left(idx);

			// Vai con la verifica!
			if( StdName( titolo))
			{
				AfxGetMainWnd()->MessageBox( "Il nome specificato per la macro e' gia' "
					"utilizzato per un componente della libreria standard.", "Errore",
					MB_ICONSTOP);
				AfxThrowArchiveException( CArchiveException::none);
				return;
			}
		}

		// Ritrova la stringa di definizione
		CString LongTitle;
		m_TitleToolBar.m_EditTL.GetWindowText( LongTitle);

		// Salva con titolo diverso per macro e documenti
		CString str;
		if( GetID() == IDR_CHILDFRAMEMACRO)
			str.Format( "[%s %s]\r\n", MACROS_HEADER, LPCSTR( LongTitle));
		else
			str.Format( "[%s %s]\r\n", DOCS_HEADER, LPCSTR( LongTitle));

		ar.WriteString( str);

		BOOL bSplit = ((CMainFrame *)AfxGetMainWnd())->m_dlgConfig.m_bSaveMacro;
		int a,n=m_compArray.GetSize();
		CPrimitive *p;
		for( a=0; a<n; a++)
		{
			p = m_compArray.GetAt(a);

			if( p->IsProper())
			{
				CString str;
				p->GetAttrib( "Text", str);

				// Se la macro e' standard, no problem; altrimenti applica o meno
				// lo splitting
				if( p->GetProp() & PROP_ISMACRO && 
					str.GetLength() > 0 && str.GetAt(0) == '~')
						p->Save( str);
				else
						p->Save( str, bSplit);

				ar.WriteString( str);
			}
		}
	}
	else
	{
		// Legge l'intero file
		CString buffer("");
		char minibuf[1000];
		UINT nRead;
		do
		{
			nRead = ar.Read( minibuf, 1000);
			buffer += CString( minibuf, nRead);
		} while ( nRead == 1000);

		// Ora puo' processarlo
		BOOL bWarning;
		DOC_TYPE t;
		CString LongTitle;
		if( !CPrimitive::ScanHeadedBuffer( buffer, &m_compArray, NULL, &LongTitle,
			&bWarning, &t) || (t!= DOC_MACRO && t!=DOC_NORMAL))
		{
			AfxGetMainWnd()->MessageBox( "Formato del file non corretto.",
				"Errore", MB_ICONSTOP);
			AfxThrowArchiveException( CArchiveException::none);
			return;
		}

		m_TitleToolBar.SetInitialTitles( LongTitle);

		// Mostra warning per errori non critici
		if( bWarning)
			AfxGetMainWnd()->MessageBox( "Sono stati incontrati uno o piu' errori in lettura. "
				"I comandi sconosciuti sono stati ignorati.", "Attenzione", MB_ICONINFORMATION);

		// Inserisce il primo step nell'undo buffer
		UndoStore();
	}
}

/////////////////////////////////////////////////////////////////////////////
// CElecDrawDoc diagnostics

#ifdef _DEBUG
void CElecDrawDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CElecDrawDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CElecDrawDoc commands

void CElecDrawDoc::OnFileSave() 
{
	if( !WarningMessage())
		return;

	CDocument::OnFileSave();
}

void CElecDrawDoc::OnFileSaveAs() 
{
	if( !WarningMessage())
		return;

	CDocument::OnFileSaveAs();
}


int CElecDrawDoc::WarningMessage()
{
	static BOOL bFirst = TRUE;
	BOOL bSplit = ((CMainFrame *)AfxGetMainWnd())->m_dlgConfig.m_bSaveMacro;
	BOOL bLizardKing = ((CMainFrame *)AfxGetMainWnd())->m_bNoWarning;

	// Se splittiamo o non e' la prima volta (o siamo re lucertola), esce
	if( bSplit || !bFirst || bLizardKing)
		return TRUE;

	// Cerca eventuali macro nonstandard nel documento
	int a, n=m_compArray.GetSize();
	CPrimitive *p;
	CString str;
	for( a=0; a<n; a++)
	{
		p = m_compArray.GetAt(a);
		p->GetAttrib( "Text", str);
		if( p->GetProp() & PROP_ISMACRO && 
			str.GetLength() > 0 && str.GetAt(0) != '~')
			break;
	}
	if( a==n)
		return TRUE;

	bFirst = FALSE;

	int res = AfxGetMainWnd()->MessageBox( "Stai per salvare un documento senza "
			"scindere le macro NON STANDARD in esso contenute (opzione impostabile "
			"nella dialog Opzioni/Configurazione). Questo puo' portare a una potenziale "
			"impossibilita' che altri utenti possano leggere correttamente il disegno. "
			"Per salvare comunque le macro non standard non splittate, premi SI; per "
			"modificare l'opzione e salvare le macro nonstandard splittate (mantenendo "
			"comunque inalterate le macro di sistema), premi NO; per interrompere questo "
			"salvataggio premi Annulla.\r\n(questo messaggio di avvertimento sara' "
			"mostrato solo questa volta).", "Attenzione", MB_ICONQUESTION | MB_YESNOCANCEL);
	
	if( res == IDNO)
		((CMainFrame *)AfxGetMainWnd())->m_dlgConfig.m_bSaveMacro = TRUE;
	else if( res == IDCANCEL)
		return FALSE;

	return TRUE;
}

void CElecDrawDoc::ReportSaveLoadException(LPCTSTR lpszPathName, CException* e, BOOL bSaving, UINT nIDPDefault) 
{
	if( e->IsKindOf( RUNTIME_CLASS(CArchiveException)))
		return;

	CDocument::ReportSaveLoadException(lpszPathName, e, bSaving, nIDPDefault);
}

BOOL CElecDrawDoc::StdName( CString& name)
{
	CString buf;
	LPSTR lpRes;
	HANDLE hResInfo, hRes;

	hResInfo = ::FindResource( NULL, MAKEINTRESOURCE(IDR_STDLIB), "STDLIBS");
	if( hResInfo != NULL)
	{
    	hRes = ::LoadResource( NULL, (HRSRC)hResInfo);
    	if (hRes != NULL)
		{
    		lpRes = (LPSTR)(::LockResource( hRes));
    		if( lpRes != NULL)
    		{
				buf = CString( lpRes);
        		::UnlockResource( hRes);
    		}
    		::FreeResource( hRes);
		}
	}

	STRARRAY lc;
	if( !CPrimitive::ScanHeadedBuffer( buf, NULL, &lc, NULL, NULL, NULL))
		return FALSE;

	// Scorre gli elementi della libreria statdard
	CString MacroBrief;
	int a, n=lc.GetSize();
	for( a=0; a<n; a++)
	{
		if( CMacro::ScanLibraryMacro( *(lc[a]), NULL, &MacroBrief, NULL, NULL) &&
				MacroBrief == name)
					break;
	}

	// Elimina la fame nel mondo
	int z, m=lc.GetSize();
	for( z=0; z<m; z++)
		delete (lc[z]);

	if( a<n)
		return TRUE;
	else
		return FALSE;
}

void CElecDrawDoc::GetRealSize( CRect& rc)
{
	// Devo trovare l'occupazione reale del disegno; faccio un'unione dei
	// bounding rectangles di tutti gli oggetti:
	int a, n=m_compArray.GetSize();
	CPrimitive *p;
	CRgn rgn;
	for( a=0; a<n; a++)
	{
		p = m_compArray[a];

		// Non considero componenti illegali o nascosti
		if( !CElecDrawView::IsComponentVisible(p) || p->GetRegion()->m_hObject == NULL)
			continue;

		if( rgn.m_hObject == NULL)
		{
			rgn.CreateRectRgn( 0, 0, 1, 1);
			rgn.CopyRgn( p->GetRegion());
		}
		else
			rgn.CombineRgn( &rgn, p->GetRegion(), RGN_OR);
	}

	if( rgn.m_hObject != NULL)
		rgn.GetRgnBox( &rc);
}

void CElecDrawDoc::UpdateBoundingRegions( CDC *pdc, PRIMARRAY& SelArray)
{
	int a, n=SelArray.GetSize();
	CPrimitive *p;
	CWaitCursor pippo;

	for( a=0; a<n; a++)
	{
		p = SelArray.GetAt(a);
		p->UpdateBoundingRgn( pdc);
	}
}


/*
	Funzioni di undo/redo
	Il meccanismo di undo/redo e` semplificato al massimo: ogni "stato" del documento e`
	rappresentato salvandolo completamente come testo in formato FidoCAD.
	
	UndoReset() resetta il buffer, con la possibilita` di impostare anche la sua dimensione;
	UndoStore() "aggiunge" un passo nell'undo buffer (vengono eliminati i passi piu` vecchi
	per mantenersi sotto alla dimensione del buffer; se il passo da solo e` piu` grande
	dell'intero buffer, vengono rimossi tutti gli altri passi e viene comunque memorizzato;
	Undo() ripristina il passo precedente, oppure non fa nulla se non ci sono piu` passi. Questa
	funzione non rimuove i passi dal buffer, ma si limita a decrementare il "redo pointer"; se
	in seguito si chiama UndoStore(), i passi dal redo pointer in poi vengono eliminati;
	Redo() ripristina lo stato precedente all'ultima chiamata di Undo() e incrementa il
	redo pointer. I passi di redo vengono eliminati nel caso che si chiami UndoStore().
	CanUndo() e CanRedo() restituiscono TRUE se ci sono passi disponibili nell'undo o redo buffer.

	Nota importante: per semplificare la gestione del codice nella CView, anziche` memorizzare
	il documento vecchio appena prima che esso venga modificato, viene salvato il nuovo documento
	gia` modificato. Cio` significa che e` necessario salvare il primo "step" gia` quando si
	carica un documento, e che l'ultimo step rappresenta sempre lo stato attuale del documento.
	Quindi se ad esempio nell'undo buffer c'e` un solo step, non c'e` niente da undo-are. Ecco
	uno specchietto dei casi "limite":

	undosize<=1, redopos=x:				no undo, no redo.
	undosize>1, redopos<2:				no undo, si` redo.
	undosize>1, 2<=redopos<undosize:	si undo, si. redo.
	undosize>1, redopos=undosize:		si undo, no redo.

	E` piu` facile da capire che da spiegare. :-)
*/

int CElecDrawDoc::UndoReset( int size)
{
	m_undobuffer.RemoveAll();
	m_redopos = 0;
	m_bufsize = UNDO_BUFFER_SIZE;
	if( size != -1)
		m_bufsize = size;
	return 1;
}

int CElecDrawDoc::UndoStore()
{
	// Serializza il documento attuale
	CString salva;
	int a,n=m_compArray.GetSize();
	CPrimitive *p;
	for( a=0; a<n; a++)
	{
		p = m_compArray.GetAt(a);

		if( p->IsProper())
		{
			CString str;
			p->GetAttrib( "Text", str);

			// Se la macro e' standard, no problem; altrimenti applica o meno
			// lo splitting
			if( p->GetProp() & PROP_ISMACRO && 
				str.GetLength() > 0 && str.GetAt(0) == '~')
					p->Save( str);
			else
					p->Save( str, FALSE);

			salva += str;
		}
	}

	// Distrugge il redo buffer
	n=m_undobuffer.GetSize();
	if( n>1)
		for( a=n-1; a>=m_redopos; a--)
			m_undobuffer.RemoveAt( a);

	// Guarda la dimensione dell'undo buffer
	int undosize=0;
	n = m_undobuffer.GetSize();
	for( a=0; a<n; a++)
		undosize += m_undobuffer[a].GetLength();

	// Rimuove i passi piu` vecchi dell'undo buffer finche` il nuovo
	// step non ci sta
	while( m_undobuffer.GetSize() != 0 && undosize+salva.GetLength() > m_bufsize)
	{
		undosize -= m_undobuffer[0].GetLength();
		m_undobuffer.RemoveAt( 0);
	}

	// Aggiunge lo step e aggiorna il redo pointer
	m_undobuffer.Add( salva);
	m_redopos = m_undobuffer.GetSize();
	return 1;
}

int CElecDrawDoc::Undo()
{
	if( !CanUndo())
		return 0;

	// Ripristiniamo lo step memorizzato in m_redopos-1
	m_redopos--;
	CString carica = "[FIDOCAD]\r\n" + m_undobuffer[m_redopos-1];
	PRIMARRAY compArray;
	BOOL bWarning;
	CString LongTitle;
	DOC_TYPE t;
	if( !CPrimitive::ScanHeadedBuffer( carica, &compArray, NULL, &LongTitle, &bWarning, &t))
	{
		// TODO: e in caso di errore?
	}

	// Azzeriamo il vecchio comparray e lo sostituiamo con quello nuovo
	int a, n=m_compArray.GetSize();
	for( a=n-1; a>=0; a--)
		delete m_compArray[a];
	m_compArray.RemoveAll();
	m_selArray.RemoveAll();

	n = compArray.GetSize();
	for( a=0; a<n; a++)
		m_compArray.Add( compArray[a]);

	return 1;
}

int CElecDrawDoc::Redo()
{
	if( !CanRedo())
		return 0;

	// Ripristiniamo lo step memorizzato in m_redopos+1
	m_redopos++;
	CString carica = "[FIDOCAD]\r\n" + m_undobuffer[m_redopos-1];
	BOOL bWarning;
	CString LongTitle;
	DOC_TYPE t;
	PRIMARRAY compArray;
	if( !CPrimitive::ScanHeadedBuffer( carica, &compArray, NULL, &LongTitle, &bWarning, &t))
	{
		// TODO: e in caso di errore?
	}

	// Azzeriamo il vecchio comparray e lo sostituiamo con quello nuovo
	int a, n=m_compArray.GetSize();
	for( a=n-1; a>=0; a--)
		delete m_compArray[a];
	m_compArray.RemoveAll();
	m_selArray.RemoveAll();

	n = compArray.GetSize();
	for( a=0; a<n; a++)
		m_compArray.Add( compArray[a]);

	return 1;
}

BOOL CElecDrawDoc::CanUndo()
{
	if( m_undobuffer.GetSize() > 1 && m_redopos > 1)
		return TRUE;
	else
		return FALSE;
}

BOOL CElecDrawDoc::CanRedo()
{
	if( m_undobuffer.GetSize() > 1 && m_redopos < m_undobuffer.GetSize())
		return TRUE;
	else
		return FALSE;
}

void CElecDrawDoc::OnEditUndo() 
{
	Undo();
	UpdateAllViews( NULL);
}

void CElecDrawDoc::OnUpdateEditUndo(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( CanUndo());
}

void CElecDrawDoc::OnEditRedo() 
{
	Redo();
	UpdateAllViews( NULL);
}

void CElecDrawDoc::OnUpdateEditRedo(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( CanRedo());
}


/*
	Per salvare il file in un bitmap, ci sono tre modi:
	
	-Relativo: si specifica uno zoom, che sara` l'unita` di conversione da UL a pixel;
	-Assoluto: si specifica una risoluzione (in dpi), che assieme alla dimensione "fisica"
		della UL servira` per determinare le dimensioni del bitmap. Da notare che purtroppo
		non tutti i programmi di fotoritocco considerano i dpi specificati nei file bmp.
	-EMF: salva nel formato Enhanced Metafile di Windows
*/

void CElecDrawDoc::OnFileSavebmp() 
{
	CDlgSaveBmp dlg;
	if( dlg.DoModal() != IDOK)
		return;

	CString filtro = dlg.m_mode==2 ? "Enhanced Metafile (*.emf)|*.emf|Tutti i file (*.*)|*.*||" :
									"Bitmap files (*.bmp)|*.bmp|Tutti i file (*.*)|*.*||";
	CFileDialog savedlg( FALSE, "bmp", NULL, OFN_HIDEREADONLY | OFN_PATHMUSTEXIST |
		OFN_OVERWRITEPROMPT, filtro, AfxGetMainWnd());
	if( savedlg.DoModal() != IDOK)
		return;

	BOOL bMirrorPCB = ((CMainFrame *)AfxGetMainWnd())->m_dlgConfig.m_bPrintMirror;
	POSITION pos = GetFirstViewPosition();
	CElecDrawView *view = (CElecDrawView *)GetNextView( pos);
	if( view == NULL)
	{
		AfxGetMainWnd()->MessageBox( "Impossibile trovare la vista!", NULL,
			MB_OK | MB_ICONERROR);
		return;
	}

	// Se la modalita` di output e` la MetaFile non ci interessano i mapping modes;
	// Disegnamo direttamente e con uno zoom logico del 2000% (vedi descrizione della
	// funzione CPrimitive::Draw in primitive.h)
	if( dlg.m_mode == 2)
	{
		CMetaFileDC mfdc;
		if( !mfdc.CreateEnhanced( NULL, savedlg.GetPathName(), NULL,
			"FidoCAD Metafile Writer\0" + GetTitle() + "\0\0"))
		{
			AfxGetMainWnd()->MessageBox( "Impossibile creare il metafile!", NULL,
				MB_OK | MB_ICONERROR);
			return;
		}

		// deseleziono tutto
		m_selArray.RemoveAll();

		int ll, lp, lc, n=m_compArray.GetSize();
		CPrimitive *p;
		for( ll=0; ll<16; ll++)
		{
			// Stampo solo i layer visibili (la considero una visualizzazione)...
			if( !(((CMainFrame*)AfxGetMainWnd())->m_bLayerShow[ll]))
				continue;

			// ...alla priorita` giusta
			for( lp=0; lp<=6; lp++)
			{
				for( lc=0; lc<n; lc++)
				{
					// Le macro passano sempre (basta che siamo nella loro priorita`) perche` potrebbero
					// contenere componenti su qualsiasi layer
					p = m_compArray.GetAt( lc);
					p->Select( FALSE);

					if( ((p->GetProp() & PROP_ISMACRO) || p->GetLayer() == ll)
							&& p->GetDrawPriority() == lp)
					{
						// Gli EMF li disegnamo a colori e ce ne freghiamo sempre dello swap
						COLORREF clr = ((CMainFrame *)AfxGetMainWnd())->m_layerColors[p->GetLayer()];

						if( p->GetProp() & PROP_ISMACRO)
							((CMacro*)p)->Draw( &mfdc, FALSE, RGB(255,255,255), clr, RGB(255,0,0), 2000, ll);
						else
							p->Draw( &mfdc, FALSE, RGB(255,255,255), clr, RGB(255,0,0), 2000);
					}
				}
			}
		}
		
		::DeleteEnhMetaFile( mfdc.CloseEnhanced());
		return;
	}

	/*
		Dobbiamo impostare il mapping mode (per il quale non ci serve conoscere la
		dimensione effettiva del disegno) prima di chiamare UpdateBoundingRegions().
	*/
	CClientDC dc( AfxGetMainWnd());
	CDC destdc;
	destdc.CreateCompatibleDC( &dc);

	int mpul = ((CMainFrame*)AfxGetMainWnd())->m_dlgConfig.m_mmPerUL;
	if( dlg.m_mode == 0)
	{
		// Modo relativo: pixel=f(UL,zoom)
		destdc.SetMapMode( MM_ISOTROPIC);
		destdc.SetWindowExt( DEFAULT_AREAX, DEFAULT_AREAY);
		destdc.SetViewportExt( (int)(DEFAULT_AREAX*dlg.m_zoom/100),
			(int)(DEFAULT_AREAY*dlg.m_zoom/100));
	}
	else
	{
		// Modo assoluto: pixel=f(UL,DPI,mmPerUL)
		destdc.SetMapMode( MM_ISOTROPIC);
		destdc.SetWindowExt( DEFAULT_AREAX, DEFAULT_AREAY);
		destdc.SetViewportExt( DEFAULT_AREAX*dlg.m_dpi*mpul/25600,
			DEFAULT_AREAY*dlg.m_dpi*mpul/25600 );
	}

	// Ora possiamo aggiornare le regioni del disegno e trovare la sua dimensione reale
	UpdateBoundingRegions( &destdc, m_compArray);
	CRect rcSize;
	GetRealSize( rcSize);
	rcSize.InflateRect( 5, 5);

	// Ora impostiamo l'origine del viewport in modo che il bordo superiore sinistro del
	// disegno si trovi al pixel (0,0)
	destdc.SetWindowOrg( rcSize.left, rcSize.top);

	// Creiamo un bitmap delle giuste dimensioni e lo selezioniamo nel dc
	int sizex, sizey;
	LONG pelsperm;
	if( dlg.m_mode == 0)
	{
		sizex = rcSize.Width()*dlg.m_zoom/100;
		sizey = rcSize.Height()*dlg.m_zoom/100;
		pelsperm = 0;
	}
	else
	{
		sizex = rcSize.Width()*dlg.m_dpi*mpul/25600;
		sizey = rcSize.Height()*dlg.m_dpi*mpul/25600;
		pelsperm = (int)(dlg.m_dpi*100/2.54);
	}
	CBitmap bmp;
	bmp.CreateBitmap( sizex, sizey, 1, 1, NULL);

	CBitmap *oldbmp = (CBitmap*)destdc.SelectObject( &bmp);
	if( oldbmp != NULL)
	{
		m_selArray.RemoveAll();
		destdc.Rectangle( -1, -1, DEFAULT_AREAX, DEFAULT_AREAY);

		int ll, lp, lc, n=m_compArray.GetSize();
		CPrimitive *p;
		for( ll=0; ll<16; ll++)
		{
			// Stampo solo i layer visibili (la considero una visualizzazione)...
			if( !(((CMainFrame*)AfxGetMainWnd())->m_bLayerShow[ll]))
				continue;

			// ...alla priorita` giusta
			for( lp=0; lp<=6; lp++)
			{
				for( lc=0; lc<n; lc++)
				{
					// Le macro passano sempre (basta che siamo nella loro priorita`) perche` potrebbero
					// contenere componenti su qualsiasi layer
					p = m_compArray.GetAt( lc);
					p->Select( FALSE);

					if( ((p->GetProp() & PROP_ISMACRO) || p->GetLayer() == ll)
							&& p->GetDrawPriority() == lp)
					{
						// I bitmap li disegnamo in bianco e nero e ce ne freghiamo dello swap
						COLORREF clr = RGB(0,0,0);

						if( p->GetProp() & PROP_ISMACRO)
							((CMacro*)p)->Draw( &destdc, FALSE, RGB(255,255,255), clr, RGB(255,0,0), 100, ll, TRUE);
						else
							p->Draw( &destdc, FALSE, RGB(255,255,255), clr);
					}
				}
			}
		}

		// Non ci resta altro che salvare i bits contenuti nel bitmap!
		BITMAPFILEHEADER bfh;
		BITMAPINFOHEADER bih;
		RGBQUAD bmiColors[2] = {{0,0,0,0},{255,255,255,0}};
		BITMAP bitmap;
		destdc.SelectObject( oldbmp);
		bmp.GetBitmap( &bitmap);

		// In certi casi (assurdo) bmWidthBytes non e multiplo di 4 come dovrebbe!
		int bmWidthBytes1 = bitmap.bmWidthBytes;
		if( bmWidthBytes1 % 4 != 0)
			bmWidthBytes1 += (4-bmWidthBytes1%4);
		int srcsize = bitmap.bmWidthBytes*bitmap.bmHeight;
		int destsize = bmWidthBytes1*bitmap.bmHeight;

		bfh.bfType = 0x4D42;
		bfh.bfSize = sizeof(bfh) + sizeof(bih) + sizeof(bmiColors) + destsize;
		bfh.bfReserved1 = bfh.bfReserved2 = 0;
		bfh.bfOffBits = sizeof(bfh) + sizeof(bih) + sizeof(bmiColors);
		bih.biSize = sizeof(bih);
		bih.biWidth = bitmap.bmWidth;
		bih.biHeight = bitmap.bmHeight;
		bih.biPlanes = bitmap.bmPlanes;
		bih.biBitCount = bitmap.bmBitsPixel;
		bih.biCompression = BI_RGB;
		bih.biSizeImage = 0;
		bih.biXPelsPerMeter = bih.biYPelsPerMeter = pelsperm;
		bih.biClrUsed = bih.biClrImportant = 0;

		BYTE* buffa = (BYTE*)malloc( srcsize);
		if( buffa != NULL)
		{
			bmp.GetBitmapBits( srcsize, (LPVOID)buffa);
			FILE *h = fopen( savedlg.GetPathName(), "wb");
			if( h != NULL)
			{
				fwrite( &bfh, sizeof(bfh), 1, h);
				fwrite( &bih, sizeof(bih), 1, h);
				fwrite( &bmiColors, sizeof(bmiColors), 1, h);

				/*
					Tanti programmi di grafica merdosi nel 2000 non leggono ancora
					i top-down DIB, quindi occorre ribaltare a mano le righe del bitmap
					per trasformarlo in un bottom-up DIB.
					Inoltre se bmWidthBytes non e` multiplo di 4, dobbiamo aggiungere i byte
					di padding.
				*/
				int a;
				BYTE pad[3] = {0,0,0};
				for( a=srcsize-bitmap.bmWidthBytes; a>=0; a -= bitmap.bmWidthBytes)
				{
					fwrite( &buffa[a], bitmap.bmWidthBytes, 1, h);
					if( bitmap.bmWidthBytes % 4 != 0)
						fwrite( pad, 4-bitmap.bmWidthBytes%4, 1, h);
				}
				fclose(h);
			}
			free( buffa);
		}
		else
			AfxGetMainWnd()->MessageBox( "Errore di allocazione di memoria!", NULL,
				MB_OK | MB_ICONERROR);
	}
	else
		AfxGetMainWnd()->MessageBox( "Errore di creazione del bitmap!", NULL,
			MB_OK | MB_ICONERROR);

	// Dobbiamo rifare l'update di tutte le view perche` abbiamo potenzialmente modificato
	// le bounding regions!
	UpdateAllViews( NULL);
}
