/*
	ElecDrawLibDoc.cpp
	Classe del documento delle librerie.
	Dato che il documento FCL l'ho aggiunto in un secondo (o terzo, o
	quarto) tempo, mi sono trovato davanti ad alcuni problemi nella sua integrazione
	con il progetto. Toccare la vista o il documento "originari" era da pazzi, cosi'
	ecco l'idea geniale: copiare "a mano" il componente di libreria selezionato nell'
	array dei componenti della classe di base. In pratica la vista non si accorgera'
	di nulla, e modifichera' il componente di libreria giusto.
	
	Ulteriore problema: come teniamo in memoria i singoli componenti? C'era una strada
	molto semplice, ma piuttosto costosa in termini di memoria (e caricamento iniziale):
	caricare SUBITO tutti i componenti di libreria tenendoli memorizzati ognuno nel
	suo bel PRIMARRAY. Non faceva per me, cosi' ho avuto l'n-esima idea: mantenere
	memorizzati i componenti in memoria come su disco, ossia in formato testo. Quando se
	ne attiva uno, esso viene interpretato al volo e subito posto nel PRIMARRAY
	del documento di base (CElecDrawDoc); quando se ne deattiva uno si fa il percorso
	contrario, ossia lo si serializza in una serie di comandi in formato testo e lo
	si mette da parte in un buffer. L'occupazione di memoria cosi' e' esigua, e tutti
	sono piu' asciutti e felici.
	Invece di un PRIMARRAY che contiene a sua volta PRIMARRAY che contengono a loro
	volta gli oggetti costituenti di ogni componente, adottero' un CArray di puntatori
	a CString che contengono la "versione testuale" di ogni componente. Cool! :-)
*/

#include "stdafx.h"
//#include "DlgSplash.h"
#include "ElecDraw.h"
#include "Primitive.h"
#include "DlgMacroBar.h"
//#include "AuxPrim.h"
#include "ViewToolbar.h"
#include "ElecDrawDoc.h"
#include "ElecDrawLibDoc.h"
#include "GridDialog.h"
#include "ElecDrawView.h"
#include "ConfigDialog.h"
#include "LayerComboBox.h"
#include "Mainfrm.h"
#include "DlgLibNum.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CElecDrawLibDoc

IMPLEMENT_DYNCREATE(CElecDrawLibDoc, CElecDrawDoc)

BEGIN_MESSAGE_MAP(CElecDrawLibDoc, CElecDrawDoc)
	//{{AFX_MSG_MAP(CElecDrawLibDoc)
	ON_COMMAND(ID_LIB_ADD, OnLibAdd)
	ON_COMMAND(ID_LIB_DELETE, OnLibDelete)
	ON_CBN_SELCHANGE(IDC_COMBO_TLM, OnComboSelChange)
	ON_CBN_DROPDOWN(IDC_COMBO_TLM, OnComboDropDown)
	ON_CBN_SELENDOK(IDC_COMBO_TLM, OnComboSelEndOK)
	ON_CBN_EDITCHANGE(IDC_COMBO_TLM, OnComboEditChange)
	ON_COMMAND(ID_LIB_INSERT, OnLibInsert)
	ON_COMMAND(ID_LIB_INSERT_BEFORE, OnLibInsertBefore)
	ON_COMMAND(ID_LIB_NUMBER, OnLibNumber)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CElecDrawLibDoc construction/destruction

CElecDrawLibDoc::CElecDrawLibDoc()
{
}

CElecDrawLibDoc::~CElecDrawLibDoc()
{
}

void CElecDrawLibDoc::DeleteContents() 
{
	// Elimina l'oggetto in fase di editing
	CElecDrawDoc::DeleteContents();
	
	// Elimina i blocchettini di memoria
	int a, l=m_libArray.GetSize();
	for( a=0; a<l; a++)
		delete (m_libArray[a]);
	m_libArray.RemoveAll();

	m_nUnnamedComp = 1;
	m_nEditIndex = 0;
	m_nLastSel = 0;
}

BOOL CElecDrawLibDoc::OnNewDocument() 
{	
	if( !CElecDrawDoc::OnNewDocument())
		return FALSE;

	// Imposta il titolo lungo della libreria
	CString LongTitle;
	LongTitle.Format( "Nuova libreria %d", ((CMainFrame*)AfxGetMainWnd())->m_nUnnamedLib++);
	if( m_TitleToolBar.m_hWnd != NULL)
		m_TitleToolBar.m_EditTL.SetWindowText( LongTitle);
	else
		m_TitleToolBar.SetInitialTitles( LongTitle);

	return AddComponent();
}

BOOL CElecDrawLibDoc::AddComponent( int nIndex)
{
	if( nIndex < 0 || nIndex >= m_libArray.GetSize())
		nIndex = -1;

	// Svuota l'undo buffer
	UndoReset();

	CString Short, Long;
	SvuotaSquassa();
	do
	{
		Short.Format("M%02d", m_nUnnamedComp);
		Long.Format("Componente %d", m_nUnnamedComp++);
	} while( FindDupTitle( Short, -1));

	CString* str = new CString;
	str->Format("[%s %s]\r\n", LPCSTR(Short), LPCSTR(Long));

	if( nIndex == -1)
	{
		m_nEditIndex = m_libArray.Add(str);
		if( m_TitleToolBar.m_hWnd != NULL)
		{
			m_TitleToolBar.m_EditTBM.SetWindowText(Short);
			m_TitleToolBar.m_ComboTLM.AddString(Long);
			m_TitleToolBar.m_ComboTLM.SetCurSel( m_nEditIndex);
			m_nLastSel = m_nEditIndex;
		}
	}
	else
	{
		m_nEditIndex = nIndex;
		m_libArray.InsertAt( nIndex, str);
		if( m_TitleToolBar.m_hWnd != NULL)
		{
			m_TitleToolBar.m_EditTBM.SetWindowText(Short);
			m_TitleToolBar.m_ComboTLM.InsertString( nIndex, Long);
			m_TitleToolBar.m_ComboTLM.SetCurSel( nIndex);
			m_nLastSel = m_nEditIndex;
		}
	}

	UpdateAllViews( NULL);
	return TRUE;
}

BOOL CElecDrawLibDoc::RemoveComponent()
{
	CString* str = m_libArray[m_nEditIndex];
	delete str;
	m_libArray.RemoveAt( m_nEditIndex);
	m_TitleToolBar.m_ComboTLM.DeleteString( m_nEditIndex);

	if( m_nEditIndex == 0)
		m_nEditIndex = 0;
	else
		m_nEditIndex--;

	if( m_libArray.GetSize() == 0)
		AddComponent();
	else
		LoadComponent();

	return TRUE;
}

BOOL CElecDrawLibDoc::SaveComponent()
{
	BOOL bFail=FALSE;
	CString Short, Long;

	m_TitleToolBar.m_EditTBM.GetWindowText( Short);
	m_TitleToolBar.m_ComboTLM.GetWindowText( Long);

	// Stringhe vuote?		
	if( Short.IsEmpty() || Long.IsEmpty())
	{
		AfxGetMainWnd()->MessageBox( "Sia il titolo breve che quello lungo "
			"devono essere specificati.");
		bFail = TRUE;
	}

	// Titolo breve invalido?
	if( InvalidShortTitle( Short) && !bFail)
	{
		AfxGetMainWnd()->MessageBox( "Il nome breve del componente di libreria "
			"deve essere composto da soli caratteri alfanumerici, e non deve "
			"essere piu' lungo di 8 caratteri.");
		bFail = TRUE;
	}
	
	// Titolo breve duplicato?
	if( FindDupTitle( Short, m_nEditIndex) && !bFail)
	{
		AfxGetMainWnd()->MessageBox( "Il nome breve del componente di libreria "
			"e' gia' utilizzato.");
		bFail = TRUE;
	}

	// Aggiorna le stringhe nella combo
	CString str;
	m_TitleToolBar.m_ComboTLM.GetWindowText( str);
	m_TitleToolBar.m_ComboTLM.DeleteString( m_nEditIndex);
	m_TitleToolBar.m_ComboTLM.InsertString( m_nEditIndex, str);

	// I titoli sono cambiati?
	CString OldShort, OldLong;
	if( !CMacro::ScanLibraryMacro( *(m_libArray[m_nEditIndex]), NULL,
			&OldShort, &OldLong, NULL))
	{
		MessageBeep(0);
		return FALSE;
	}
	
	if( !bFail)
	{
		if( OldLong != Long || OldShort != Short)
			SetModifiedFlag();
	}
	else
	{
		Long = OldLong;
		Short = OldShort;
	}

	// Serializza il componente che "esce"
	int a, n=m_compArray.GetSize();
	m_libArray[m_nEditIndex]->Format("[%s %s]\r\n", LPCSTR(Short), LPCSTR(Long));
	CString buf;
	for( a=0; a<n; a++)
	{
		m_compArray[a]->Save( buf);
		*(m_libArray[m_nEditIndex]) += buf;
	}

	if( bFail)
		return FALSE;
	else
		return TRUE;
}

BOOL CElecDrawLibDoc::LoadComponent( BOOL bUpdate)
{
	// Svuota l'undo buffer
	UndoReset();

	// Deserializza il componente entrante
	CString Short, Long;
	CMacro container;
	if( CMacro::ScanLibraryMacro( *(m_libArray[m_nEditIndex]), &container,
			&Short, &Long, NULL))
	{
		SvuotaSquassa();

		PRIMARRAY* tempArray = container.GetComponentArray();

		int a, n = tempArray->GetSize();
		if( n!=0)
		{
			for( a=0; a<n; a++)
				m_compArray.Add( tempArray->GetAt(a)->Clone());

			// Fa l'update delle regions
			POSITION pos = GetFirstViewPosition();
			CElecDrawView* view = (CElecDrawView *)GetNextView( pos);
			if( view != NULL)
			{
				CClientDC dc( view);
				view->OnPrepareDC( &dc);
				UpdateBoundingRegions( &dc, m_compArray);
			}
		}

		if( m_TitleToolBar.m_hWnd != NULL)
		{
			m_TitleToolBar.m_EditTBM.SetWindowText(Short);
			m_TitleToolBar.m_ComboTLM.SetWindowText(Long);
			m_TitleToolBar.m_ComboTLM.SetCurSel( m_nEditIndex);
			m_nLastSel = m_nEditIndex;
		}

		if( bUpdate)
		{
			//UpdateAllViews( NULL);
			POSITION pos = GetFirstViewPosition();
			CElecDrawView* view = (CElecDrawView *)GetNextView( pos);
			if( view != NULL)
				view->OnViewFit();
		}

		// Inserisce il primo step nell'undo buffer
		UndoStore();

		return TRUE;
	}
	else
	{
		MessageBeep(0);
		return FALSE;
	}
}

/////////////////////////////////////////////////////////////////////////////
// CElecDrawLibDoc commands

void CElecDrawLibDoc::OnComboSelChange()
{
	int idx = m_TitleToolBar.m_ComboTLM.GetCurSel();

	// Se abbiamo selezionato il componente attivo, ignora tutto
	if( idx == m_nLastSel)
		return;

	if( m_nLastSel != -1)
		m_nEditIndex = idx;

	if( LoadComponent())
	{
		POSITION pos = GetFirstViewPosition();
		CElecDrawView* view = (CElecDrawView *)GetNextView( pos);
		if( view != NULL && view->IsFitting())
			view->FoolSize();
	}

	m_TitleToolBar.m_ComboTLM.SetCurSel( m_nEditIndex);
	m_nLastSel = m_nEditIndex;
}

void CElecDrawLibDoc::OnComboSelEndOK()
{
	// Se abbiamo selezionato il componente attivo, ignora tutto
	int idx = m_TitleToolBar.m_ComboTLM.GetCurSel();
	if( idx == m_nLastSel)
		return;

	// Salva il componente uscente
	if( !SaveComponent())
	{
		m_TitleToolBar.m_ComboTLM.SetCurSel( m_nEditIndex);
		m_nLastSel = m_nEditIndex;
	}
}

void CElecDrawLibDoc::OnComboDropDown()
{
	// Imposta la stringa attuale
	CString str;
	m_TitleToolBar.m_ComboTLM.GetWindowText( str);
	m_TitleToolBar.m_ComboTLM.DeleteString( m_nEditIndex);
	m_TitleToolBar.m_ComboTLM.InsertString( m_nEditIndex, str);
	m_TitleToolBar.m_ComboTLM.SetCurSel( m_nEditIndex);
	m_nLastSel = m_nEditIndex;
}

void CElecDrawLibDoc::OnComboEditChange()
{
	m_nLastSel = -1;
	SetModifiedFlag();
}

void CElecDrawLibDoc::OnLibAdd() 
{
	if( SaveComponent())
		if( AddComponent())
		{
			POSITION pos = GetFirstViewPosition();
			CElecDrawView* view = (CElecDrawView *)GetNextView( pos);
			if( view != NULL && view->IsFitting())
				view->FoolSize();

			SetModifiedFlag();
		}
}

void CElecDrawLibDoc::OnLibInsert()		// Inserisci dopo
{
	if( SaveComponent())
		if( AddComponent( m_nEditIndex+1))
		{
			POSITION pos = GetFirstViewPosition();
			CElecDrawView* view = (CElecDrawView *)GetNextView( pos);
			if( view != NULL && view->IsFitting())
				view->FoolSize();

			SetModifiedFlag();
		}
}

void CElecDrawLibDoc::OnLibInsertBefore() 
{
	if( SaveComponent())
		if( AddComponent( m_nEditIndex))
		{
			POSITION pos = GetFirstViewPosition();
			CElecDrawView* view = (CElecDrawView *)GetNextView( pos);
			if( view != NULL && view->IsFitting())
				view->FoolSize();

			SetModifiedFlag();
		}
}

void CElecDrawLibDoc::OnLibDelete() 
{
	if( RemoveComponent())
	{
		POSITION pos = GetFirstViewPosition();
		CElecDrawView* view = (CElecDrawView *)GetNextView( pos);
		if( view != NULL && view->IsFitting())
			view->FoolSize();

		SetModifiedFlag();
	}
}

void CElecDrawLibDoc::OnLibNumber() 
{
	// Aggiorna il componente attuale
	SaveComponent();

	// Dialogwarez
	CDlgLibNum dlg;
	dlg.SetLibArray( &m_libArray);
	if( dlg.DoModal())
	{
		SetModifiedFlag();
		LoadComponent();
	}
}


///////////////////////////////////////////////////////////////////////////


BOOL CElecDrawLibDoc::InvalidShortTitle( CString& Short)
{
	int n=Short.GetLength();
	if( n<1 || n>8)
		return TRUE;

	CString charset("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_");
	CString c1 = Short.SpanExcluding(charset);
	
	if( c1 != "")
		return TRUE;
	else
		return FALSE;
}

BOOL CElecDrawLibDoc::FindDupTitle( CString& str, int nExclude)
{
	int a, n=m_libArray.GetSize();
	CString st;
	for( a=0; a<n; a++)
	{
		if( a != nExclude)
		{
			if( CMacro::ScanLibraryMacro( *(m_libArray[a]), NULL, &st, NULL,
					NULL) && st == str)
						return TRUE;
		}
	}

	return FALSE;
}

void CElecDrawLibDoc::SvuotaSquassa()
{
	// Svuota il compArray e il selArray, nonche' i titoli brevi/lunghi della macro
	int a, n = m_compArray.GetSize();
	for( a=0; a<n; a++)
		delete m_compArray[a];
	m_compArray.RemoveAll();
	m_selArray.RemoveAll();
	if( m_TitleToolBar.m_hWnd != NULL)
	{
		m_TitleToolBar.m_EditTBM.SetWindowText( "");
		m_TitleToolBar.m_ComboTLM.SetWindowText( "");
	}
}

/////////////////////////////////////////////////////////////////////////////
// CElecDrawLibDoc serialization

void CElecDrawLibDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// Salva il componente attualmente in editing
		if( !SaveComponent())
			AfxThrowArchiveException( CArchiveException::none);

		LoadComponent();

		// Ritrova e scrive la stringa di definizione nell'header
		CString str;
		CString LongTitle;
		m_TitleToolBar.m_EditTL.GetWindowText( LongTitle);
		str.Format( "[%s %s]\r\n", LIBS_HEADER, LPCSTR( LongTitle));
		ar.WriteString( str);

		// Vai di salvataggio (semplice, vero?)
		int a,n=m_libArray.GetSize();
		for( a=0; a<n; a++)
			ar.WriteString( LPCSTR(*(m_libArray[a])));
	}
	else
	{
		// Legge l'intero file
		CString buffer("");
		char minibuf[1000];
		UINT nRead;
		do
		{
			nRead = ar.Read( minibuf, 1000);
			buffer += CString( minibuf, nRead);
		} while ( nRead == 1000);

		// Ora puo' processarlo
		DOC_TYPE t;
		CString LongTitle;
		BOOL bWarning;
		if( !CPrimitive::ScanHeadedBuffer( buffer, NULL, &m_libArray, &LongTitle,
			&bWarning, &t) || t!=DOC_LIBRARY)
		{
			AfxGetMainWnd()->MessageBox( "Formato del file non corretto.",
				"Errore", MB_ICONSTOP);
			AfxThrowArchiveException( CArchiveException::none);
			return;
		}

		m_TitleToolBar.SetInitialTitles( LongTitle);

		// Mostra warning per errori non critici
		if( bWarning)
			AfxGetMainWnd()->MessageBox( "Sono stati incontrati uno o piu' errori in lettura. "
				"I comandi sconosciuti sono stati ignorati.", "Attenzione", MB_ICONINFORMATION);

		// Imposta come selezionato il primo componente
		LoadComponent( FALSE);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CElecDrawLibDoc diagnostics

#ifdef _DEBUG
void CElecDrawLibDoc::AssertValid() const
{
	CElecDrawDoc::AssertValid();
}

void CElecDrawLibDoc::Dump(CDumpContext& dc) const
{
	CElecDrawDoc::Dump(dc);
}
#endif //_DEBUG
