// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
//#include "DlgSplash.h"
#include "ElecDraw.h"
#include "Primitive.h"
#include "DlgMacroBar.h"
#include "ConfigDialog.h"
#include "GridDialog.h"
#include "ViewToolbar.h"
#include "ElecDrawDoc.h"
#include "ElecDrawView.h"
#include "LayerComboBox.h"
#include "MainFrm.h"
#include "Splash.h"
#include "DlgLayers.h"
#include "DlgLibBuf.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CMDIFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_COMMAND(ID_VIEW_PRIMITIVE_BAR, OnViewPrimitiveBar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_PRIMITIVE_BAR, OnUpdateViewPrimitiveBar)
	ON_COMMAND(ID_BUTTON_SELECT, OnButtonSelect)
	ON_COMMAND(ID_BUTTON_LINE, OnButtonLine)
	ON_COMMAND(ID_BUTTON_BEZIER, OnButtonBezier)
	ON_COMMAND(ID_BUTTON_POLY, OnButtonPoly)
	ON_WM_CLOSE()
	ON_WM_ENDSESSION()
	ON_COMMAND(ID_BUTTON_CONN, OnButtonConn)
	ON_COMMAND(ID_VIEW_REFRESH_MACRO, OnViewRefreshMacro)
	ON_COMMAND(ID_BUTTON_MACRO, OnButtonMacro)
	ON_UPDATE_COMMAND_UI(ID_BUTTON_MACRO, OnUpdateButtonMacro)
	ON_COMMAND(ID_OPTIONS_CONFIG, OnOptionsConfig)
	ON_COMMAND(ID_VIEW_MACRO_BAR, OnViewMacroBar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_MACRO_BAR, OnUpdateViewMacroBar)
	ON_COMMAND(ID_BUTTON_ZOOM, OnButtonZoom)
	ON_COMMAND(ID_BUTTON_ELLIPSE, OnButtonEllipse)
	ON_COMMAND(ID_BUTTON_RECTANGLE, OnButtonRectangle)
	ON_COMMAND(ID_EDIT_PASTENEW, OnEditPastenew)
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTENEW, OnUpdateEditPastenew)
	ON_COMMAND(ID_BUTTON_DRAG, OnButtonDrag)
	ON_COMMAND(ID_BUTTON_EXTENDEDTEXT, OnButtonExtendedtext)
	ON_COMMAND(ID_VIEW_LAYERS, OnViewLayers)
	ON_COMMAND(ID_BUTTON_PAD, OnButtonPad)
	ON_COMMAND(ID_BUTTON_VIA, OnButtonVia)
	ON_CBN_SELCHANGE(IDC_COMBO_LAYER, OnLayerChange)
	//}}AFX_MSG_MAP
	// Global help commands
	ON_COMMAND(ID_HELP_FINDER, CFrameWnd::OnHelpFinder)
	ON_COMMAND(ID_HELP, CFrameWnd::OnHelp)
	ON_COMMAND(ID_CONTEXT_HELP, CFrameWnd::OnContextHelp)
	ON_COMMAND(ID_DEFAULT_HELP, CFrameWnd::OnHelpFinder)
	ON_NOTIFY( NM_CLICK, IDC_TREE_MACRO, OnMacroSelChange)
	ON_NOTIFY( TVN_SELCHANGED, IDC_TREE_MACRO, OnMacroSelChange)
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	m_bNoWarning = m_bRegionDebug = FALSE;
	m_nUnnamedLib = 1;
	m_CacheUpdateThread = NULL;
	m_bDestroyCacheThread = FALSE;

	// Defaults per le impostazioni dei layer
	LayerDefaultNames();
	LayerDefaultColors();
	LayerDefaultShow();
	LayerDefaultPrint();

	m_currentLayer = 0;
}

void CMainFrame::LayerDefaultNames()
{
	CString names[] = { "Schema", "PCB lato rame", "PCB lato componenti", "Serigrafie" };
	int a;
	for( a=0; a<4; a++)
		m_layerNames[a] = names[a];
	for( ; a<16; a++)
		m_layerNames[a].Format( "Layer %d", a+1);
}

void CMainFrame::LayerDefaultColors()
{
	COLORREF cols[] = { RGB(0,0,0), RGB(0,0,192), RGB(0,192,0), RGB(0,150,150) };
	int a;
	for( a=0; a<4; a++)
		m_layerColors[a] = cols[a];
	for( ; a<16; a++)
		m_layerColors[a] = RGB(0,0,0);
}

void CMainFrame::LayerDefaultShow()
{
	int a;
	for( a=0; a<16; a++)
		m_bLayerShow[a] = TRUE;
}

void CMainFrame::LayerDefaultPrint()
{
	int a;
	for( a=0; a<16; a++)
		m_bLayerPrint[a] = TRUE;
}

void CMainFrame::LayerLoadSettings()
{
	CString key( "Layers");

	int a,b;
	CString s;
	CString value;
	for( a=0; a<16; a++)
	{
		value.Format( "Name%d", a);
		s = AfxGetApp()->GetProfileString( key, value, "");
		if( s != "") m_layerNames[a] = s;

		value.Format( "Color%d", a);
		b = AfxGetApp()->GetProfileInt( key, value, -1);
		if( b != -1) m_layerColors[a] = (COLORREF)b;

		value.Format( "Show%d", a);
		b = AfxGetApp()->GetProfileInt( key, value, -1);
		if( b != -1) m_bLayerShow[a] = (BOOL)b;

		value.Format( "Print%d", a);
		b = AfxGetApp()->GetProfileInt( key, value, -1);
		if( b != -1) m_bLayerPrint[a] = (BOOL)b;
	}
}

void CMainFrame::LayerSaveNames()
{
	CString key( "Layers");

	int a;
	CString value;
	for( a=0; a<16; a++)
	{
		value.Format( "Name%d", a);
		AfxGetApp()->WriteProfileString( key, value, m_layerNames[a]);
	}
}

void CMainFrame::LayerSaveColors()
{
	CString key( "Layers");

	int a;
	CString value;
	for( a=0; a<16; a++)
	{
		value.Format( "Color%d", a);
		AfxGetApp()->WriteProfileInt( key, value, (int)m_layerColors[a]);
	}
}

void CMainFrame::LayerSaveShow()
{
	CString key( "Layers");

	int a;
	CString value;
	for( a=0; a<16; a++)
	{
		value.Format( "Show%d", a);
		AfxGetApp()->WriteProfileInt( key, value, (int)m_bLayerShow[a]);
	}
}

void CMainFrame::LayerSavePrint()
{
	CString key( "Layers");

	int a;
	CString value;
	for( a=0; a<16; a++)
	{
		value.Format( "Print%d", a);
		AfxGetApp()->WriteProfileInt( key, value, (int)m_bLayerPrint[a]);
	}
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CMDIFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	// CG: The following line was added by the Splash Screen component.
	CSplashWnd::ShowSplashScreen(this);

	// Carica le opzioni di configurazione
	LoadConfigOptions();

	// Carica impostazioni layer
	LayerLoadSettings();

	// Carica nomi librerie da bufferizzare
	LoadLibNames();

	// Crea le toolbars	
	if( !m_wndToolBar.Create(this, WS_CHILD | WS_VISIBLE | CBRS_SIZE_DYNAMIC |
			CBRS_TOP | CBRS_TOOLTIPS | CBRS_FLYBY, AFX_IDW_TOOLBAR) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar 1\n");
		return -1;      // fail to create
	}

	if (!m_wndPrimitiveBar.Create(this, WS_CHILD | WS_VISIBLE | CBRS_SIZE_DYNAMIC |
			CBRS_TOP | CBRS_TOOLTIPS | CBRS_FLYBY, AFX_IDW_TOOLBAR+10) ||
		!m_wndPrimitiveBar.LoadToolBar(IDR_PRIMITIVES))
	{
		TRACE0("Failed to create toolbar 2\n");
		return -1;      // fail to create
	}

	if( !m_wndDlgMacroBar.Create( this, IDD_MACROBAR_SMALL+m_dlgConfig.m_nBarSize,
		CBRS_RIGHT | CBRS_TOOLTIPS | CBRS_SIZE_DYNAMIC, AFX_IDW_TOOLBAR+11))
	{
		TRACE0("Failed to create dialogbar 1\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

/**************************************************************************************************
	Creazione della combo del layer nella primitive bar
**************************************************************************************************/

	// Misura la larghezza del pulsante (sara' un riferimento)
	CRect rcb;
	m_wndPrimitiveBar.GetItemRect( 0, &rcb);

	// Trova le misure del font di sistema e vede se lo puo' sfruttare
	CFont sysfont;
	sysfont.Attach( ::GetStockObject( SYSTEM_FONT));
	LOGFONT lf;
	sysfont.GetLogFont( &lf);
	if( lf.lfHeight < 0)
		lf.lfHeight *= -1;

	// Font per le combo
	m_FontLittle.CreateFont( MINIMO(lf.lfHeight,rcb.Height()-8), 0, 0, 0, FW_THIN, FALSE, FALSE, 0,
		DEFAULT_CHARSET, OUT_TT_PRECIS, CLIP_TT_ALWAYS, DEFAULT_QUALITY, DEFAULT_PITCH |
		TMPF_TRUETYPE, "Arial");

	// Dimensiona il placeholder per la combo
	m_wndPrimitiveBar.SetButtonInfo( NUM_PRIMITIVES+1, IDC_COMBO_LAYER, TBBS_SEPARATOR, rcb.Width()*7);

	// Crea la combo
	CRect r;
	m_wndPrimitiveBar.GetItemRect( NUM_PRIMITIVES+1, &r);
	int yy = r.Height() - (::GetSystemMetrics( SM_CYEDGE)+1)*2;
	m_comboLayer.SetHeight( yy);
	r.bottom = r.top+r.Height()*10;
	m_comboLayer.Create( CBS_DROPDOWNLIST | CBS_OWNERDRAWFIXED | CBS_HASSTRINGS |
		WS_VISIBLE | WS_VSCROLL | WS_TABSTOP | WS_CHILD | WS_BORDER, r, &m_wndPrimitiveBar, IDC_COMBO_LAYER);
	m_comboLayer.SetItemHeight( -1, yy);
	m_comboLayer.SetFont( &m_FontLittle);

	// Imposta i valori possibili nella combo
	int a, id;
	for( a=0; a<16; a++)
	{
		id = m_comboLayer.AddString( m_layerNames[a]);
		m_comboLayer.SetItemData( id, a+1);
	}

	// Seleziona il layer zero
	m_comboLayer.SelectString( -1, m_layerNames[0]);


/***********************************************************************************************/

	m_wndToolBar.SetWindowText( "Funzioni comuni");
	m_wndToolBar.EnableDocking( CBRS_ALIGN_ANY);

	m_wndPrimitiveBar.SetWindowText( "Primitive di disegno");
	m_wndPrimitiveBar.EnableDocking( CBRS_ALIGN_TOP | CBRS_ALIGN_BOTTOM);

	m_wndDlgMacroBar.SetWindowText( "Macro");
	m_wndDlgMacroBar.EnableDocking( CBRS_ALIGN_LEFT | CBRS_ALIGN_RIGHT);

	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);
	DockControlBar(&m_wndPrimitiveBar);
	DockControlBar(&m_wndDlgMacroBar);

	// Carica la posizione delle barre
	LoadBarState( "Toolbars");

	for( a=0; a<NUM_PRIMITIVES; a++)
	{
		if( m_wndPrimitiveBar.GetButtonStyle( a) != TBBS_SEPARATOR)
			m_wndPrimitiveBar.SetButtonStyle( a, TBBS_CHECKGROUP);

		if( a==0)
		{
			id = m_wndPrimitiveBar.GetItemID( a);
			m_wndPrimitiveBar.GetToolBarCtrl().CheckButton( id, TRUE);
		}
	}

	m_SelectedPrimitive = m_OldPrimitive = B_NONE;

	// Rintraccia la directory in cui risiede l'eseguibile
	CString helppath( AfxGetApp()->m_pszHelpFilePath);
	int idx = helppath.ReverseFind( '\\');
	if( idx != -1)
	{
		_BaseDir = helppath.Left(idx);
		if( _BaseDir.GetLength() <= 2)
			_BaseDir += CString("\\");
	}

	// Carica le macro disponibili
	LoadMacros();

	// Inizia il thread di monitorizzazione di cambiamenti di librerie/macro
	m_CacheUpdateThread = AfxBeginThread( CacheUpdateThread, (LPVOID)this);

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !(CFrameWnd::PreCreateWindow(cs)))
		return FALSE;

	CString key( "Window State");

	CRect rc;
	rc.top = AfxGetApp()->GetProfileInt( key, "top", 20);
	rc.bottom = AfxGetApp()->GetProfileInt( key, "bottom", 600);
	rc.left = AfxGetApp()->GetProfileInt( key, "left", 20);
	rc.right = AfxGetApp()->GetProfileInt( key, "right", 800);
	cs.x = rc.left;
	cs.y = rc.top;
	cs.cx = rc.right-rc.left;
	cs.cy = rc.bottom-rc.top;

	cs.dwExStyle &= ~WS_EX_CLIENTEDGE;
	return TRUE;
}

void CMainFrame::RecalcLayout( BOOL bNotify) 
{
	static BOOL bOldFloating = m_wndDlgMacroBar.IsFloating();

	CMDIFrameWnd::RecalcLayout( bNotify);

	if( !::IsWindow( m_wndDlgMacroBar.m_hWnd))
		return;

	if( m_wndDlgMacroBar.IsFloating())
	{
		if( !bOldFloating)
			DestroyAndReload();
	}
	else
	{
		CRect rc1;
		GetClientRect( &rc1);
		ClientToScreen( &rc1);
		CRect rc2;
		m_wndDlgMacroBar.GetWindowRect( &rc2);
		CRect rc3;
		m_wndStatusBar.GetWindowRect( &rc3);

		if( m_wndStatusBar.IsWindowVisible())
			rc2.bottom = rc3.bottom-40;
		else
			rc2.bottom = rc1.bottom-40;

		m_wndDlgMacroBar.SetWindowPos( NULL, 0, 0, rc2.Width(), rc2.Height(),
			SWP_NOZORDER | SWP_NOMOVE);
		rc2.InflateRect( -5, -5);
		m_wndDlgMacroBar.GetDlgItem( IDC_TREE_MACRO)->SetWindowPos( NULL, 0, 0,
			rc2.Width(), rc2.Height(), SWP_NOZORDER | SWP_NOMOVE);
	}

	bOldFloating = m_wndDlgMacroBar.IsFloating();
}


/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

void CMainFrame::OnViewPrimitiveBar() 
{
	ShowControlBar( &m_wndPrimitiveBar, (m_wndPrimitiveBar.GetStyle() &
		WS_VISIBLE) == 0, FALSE);
}

void CMainFrame::OnUpdateViewPrimitiveBar(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(( m_wndPrimitiveBar.GetStyle() & WS_VISIBLE) ? 1 : 0);
}

void CMainFrame::OnViewMacroBar() 
{
	if( m_wndDlgMacroBar.GetStyle() & WS_VISIBLE)
		ShowControlBar( &m_wndDlgMacroBar, FALSE, FALSE);
	else
		DestroyAndReload();
}

void CMainFrame::OnUpdateViewMacroBar(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(( m_wndDlgMacroBar.GetStyle() & WS_VISIBLE) ? 1 : 0);
}

void CMainFrame::OnButtonSelect() 
{
	m_OldPrimitive = m_SelectedPrimitive = B_NONE;
}

void CMainFrame::OnButtonZoom() 
{
	m_OldPrimitive = m_SelectedPrimitive = B_ZOOM;
}

void CMainFrame::OnButtonDrag() 
{
	m_OldPrimitive = m_SelectedPrimitive = B_DRAG;
}

void CMainFrame::OnButtonLine() 
{
	m_OldPrimitive = m_SelectedPrimitive = B_LINE;
}

void CMainFrame::OnButtonExtendedtext() 
{
	m_OldPrimitive = m_SelectedPrimitive = B_EXTENDEDTEXT;
}

void CMainFrame::OnButtonConn() 
{
	m_OldPrimitive = m_SelectedPrimitive = B_CONNECTION;
}

void CMainFrame::OnButtonBezier() 
{
	m_OldPrimitive = m_SelectedPrimitive = B_BEZIER;
}

void CMainFrame::OnButtonPoly() 
{
	m_OldPrimitive = m_SelectedPrimitive = B_POLY;
}

void CMainFrame::OnButtonEllipse() 
{
	m_OldPrimitive = m_SelectedPrimitive = B_ELLIPSE;
}

void CMainFrame::OnButtonRectangle() 
{
	m_OldPrimitive = m_SelectedPrimitive = B_RECTANGLE;
}

void CMainFrame::OnButtonMacro() 
{
	m_OldPrimitive = m_SelectedPrimitive = B_MACRO;
}

void CMainFrame::OnButtonVia() 
{
	m_OldPrimitive = m_SelectedPrimitive = B_VIA;
}

void CMainFrame::OnButtonPad() 
{
	m_OldPrimitive = m_SelectedPrimitive = B_PAD;
}

void CMainFrame::OnMacroSelChange(NMHDR* nmh, LRESULT* res)
{
	// Se e' stata selezionata una macro valida, attiva il pulsante delle macro nella
	// primitivebar
	if( m_wndDlgMacroBar.GetSelectedMacro() != NULL)
	{
		if( m_wndPrimitiveBar.GetToolBarCtrl().m_hWnd != NULL)
		{
			m_SelectedPrimitive = B_MACRO;
			m_wndPrimitiveBar.GetToolBarCtrl().CheckButton( ID_BUTTON_MACRO, TRUE);
		}
	}
}

void CMainFrame::OnClose() 
{
	ExitProcedure();
	CMDIFrameWnd::OnClose();
}

void CMainFrame::OnEndSession(BOOL bEnding) 
{
	ExitProcedure();
	CMDIFrameWnd::OnEndSession(bEnding);
}

void CMainFrame::ExitProcedure()
{
	SaveConfigOptions();
	SaveWindowPos();
	SaveBarState( "Toolbars");

	// m_wndDlgMacroBar.WipeOut();

	// Elimina i thread in corso
	if( m_CacheUpdateThread != NULL)
	{
		int nTries = 0;
		m_bDestroyCacheThread = TRUE;
		while( m_CacheUpdateThread != NULL && nTries < 10)
		{
			Sleep( 100);
			nTries++;
		}
	}
}

void CMainFrame::SaveWindowPos()
{
	CString key( "Window State");

	ShowWindow( SW_SHOWNORMAL);

	CRect rc;
	GetWindowRect( &rc);
	AfxGetApp()->WriteProfileInt( key, "top", rc.top);
	AfxGetApp()->WriteProfileInt( key, "bottom", rc.bottom);
	AfxGetApp()->WriteProfileInt( key, "left", rc.left);
	AfxGetApp()->WriteProfileInt( key, "right", rc.right);
}

void CMainFrame::SaveConfigOptions()
{
	CString key( "Configurazione");
	AfxGetApp()->WriteProfileInt( key, "bSplit", m_dlgConfig.m_bSaveMacro);
	AfxGetApp()->WriteProfileInt( key, "nMacrobarSize", m_dlgConfig.m_nBarSize);
	AfxGetApp()->WriteProfileInt( key, "bRegisterFileTypes", m_dlgConfig.m_bRegFiles);
	AfxGetApp()->WriteProfileInt( key, "bPrintMirror", m_dlgConfig.m_bPrintMirror);
}

void CMainFrame::LoadConfigOptions()
{
	CString key( "Configurazione");

	// Ufficiali
	m_dlgConfig.m_bSaveMacro = AfxGetApp()->GetProfileInt( key, "bSplit", TRUE);
	m_dlgConfig.m_nBarSize = AfxGetApp()->GetProfileInt( key, "nMacrobarSize", 1);
	m_dlgConfig.m_bRegFiles = AfxGetApp()->GetProfileInt( key, "bRegisterFileTypes", TRUE);
	m_dlgConfig.m_bPrintMirror = AfxGetApp()->GetProfileInt( key, "bPrintMirror", TRUE);

	// Segrete
	m_bNoWarning = AfxGetApp()->GetProfileInt( key, "bWarningYouSuck", FALSE);
	m_bRegionDebug = AfxGetApp()->GetProfileInt( key, "bRegionDebug", FALSE);
}

void CMainFrame::LoadMacros( void)
{
	m_wndDlgMacroBar.ReloadAll();
}

void CMainFrame::OnViewRefreshMacro() 
{
	LoadMacros();
}

void CMainFrame::OnUpdateButtonMacro(CCmdUI* pCmdUI) 
{
	if( m_wndDlgMacroBar.GetSelectedMacro() != NULL)
		pCmdUI->Enable( TRUE);
	else
		pCmdUI->Enable( FALSE);
}

void CMainFrame::OnOptionsConfig() 
{
	if( m_dlgConfig.DoModal() == IDOK && m_wndDlgMacroBar.GetStyle() & WS_VISIBLE)
		// Ridimensiona la macrobar (distruggendola e ricreandola)
		DestroyAndReload();
}

void CMainFrame::DestroyAndReload()
{
	BOOL bFloating = m_wndDlgMacroBar.IsFloating();
	CRect rc;
	m_wndDlgMacroBar.GetWindowRect( &rc);

	ShowControlBar( &m_wndDlgMacroBar, FALSE, FALSE);
	if( !bFloating)
		FloatControlBar( &m_wndDlgMacroBar, CPoint(0,0));	
	m_wndDlgMacroBar.DestroyWindow();

	if( !m_wndDlgMacroBar.Create( this,
		IDD_MACROBAR_SMALL+m_dlgConfig.m_nBarSize,
		CBRS_RIGHT | CBRS_TOOLTIPS | CBRS_SIZE_DYNAMIC, AFX_IDW_TOOLBAR+11))
	{
		TRACE0("Failed to create dialogbar 1\n");
		return;      // fail to create
	}
	m_wndDlgMacroBar.SetWindowText( "Macro");
	m_wndDlgMacroBar.EnableDocking( CBRS_ALIGN_LEFT | CBRS_ALIGN_RIGHT);
	LoadMacros();

	if( bFloating)
		FloatControlBar( &m_wndDlgMacroBar, CPoint( rc.left, rc.top));
	else
		DockControlBar( &m_wndDlgMacroBar);

	ShowControlBar( &m_wndDlgMacroBar, TRUE, FALSE);
}

void CMainFrame::OnUpdateEditPastenew(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(	IsClipboardFormatAvailable( CF_TEXT));
}

void CMainFrame::OnEditPastenew() 
{
	CWaitCursor pippo;
	static int nDocNumber = 0;

	/*
		Il metodo piu` pulito per incollare in un nuovo documento e` chiamare la
		Serialize() del documento. Copiamo il contenuto della clipboard in un file temporaneo
		e poi lo carichiamo.
	*/

	// Carica i dati della clipboard
	if( !OpenClipboard()) return;
	HANDLE h;
	void *p;
	if( (h = ::GetClipboardData( CF_TEXT)) == NULL || (p = ::GlobalLock( h)) == NULL) return;
	CString clip( (char *)p);
	::GlobalUnlock( h);
	::CloseClipboard();

	/*
		Cerca di capire di che tipo di documento si tratta
		(se non lo riconosce fa un altro tentativo appendendo l'header
		al testo incollato)
	*/
	DOC_TYPE type = CPrimitive::GetType( clip);
	if( type == DOC_NONE)
		clip = "[FIDOCAD]\r\n" + clip;

	// Li salva in un file temporaneo
	char szTempPath[1024];
	GetTempPath( 1024, szTempPath);
	CString strTmpFile;
	strTmpFile.Format( "%s%stmpFile%d.%s", szTempPath,
		(szTempPath[lstrlen(szTempPath)-1] != '\\') ? "\\" : "",
		nDocNumber++,
		(type == DOC_LIBRARY) ? "fcl" : (type == DOC_MACRO) ? "fcm" : "fcd");
	CFile tmpFile( strTmpFile, CFile::modeCreate | CFile::modeReadWrite | CFile::shareExclusive);
	tmpFile.Write( clip, clip.GetLength());
	tmpFile.Close();

	// Simula un normale caricamento
	CDocument *pDoc = AfxGetApp()->OpenDocumentFile( strTmpFile);
	::DeleteFile( strTmpFile);
}

UINT CMainFrame::CacheUpdateThread( LPVOID pParam )
{
	CMainFrame *mainframe = (CMainFrame*)pParam;

	HANDLE hNotif = INVALID_HANDLE_VALUE;
	CString watchdir = _BaseDir;
	watchdir.MakeUpper();

	hNotif = ::FindFirstChangeNotification( LPCSTR(watchdir), TRUE, 
			FILE_NOTIFY_CHANGE_FILE_NAME | FILE_NOTIFY_CHANGE_LAST_WRITE |
			FILE_NOTIFY_CHANGE_DIR_NAME);
	if( hNotif == INVALID_HANDLE_VALUE || hNotif == NULL)
	{
		MessageBeep(MB_ICONEXCLAMATION);
		return (UINT)(-1);
	}

	// Loop di monitorizzazione principale
	DWORD dwRes;
	while( !mainframe->m_bDestroyCacheThread)
	{
		// Aspetta per un po' le modifiche, se non sono avvenute ricomincia
		dwRes = ::WaitForSingleObject( hNotif, 200);
		if( dwRes == WAIT_TIMEOUT || dwRes == WAIT_FAILED || dwRes == WAIT_ABANDONED)
			continue;

		// Elimina dalla cache TUTTI i componenti nonstandard
		CPrimitive *prim, *primt;
		CString name;
		CString namet;
		POSITION p = _MacroCache.GetStartPosition();
		while( p != NULL)
		{
			_MacroCache.GetNextAssoc( p, name, (CObject*&)prim);
			if( name[0] == '~')
				continue;

			namet = CString("~") + name;
			if( _MacroCache.Lookup( namet, (CObject*&)primt))
				continue;

			_MacroCache.RemoveKey( name);
			delete prim;
		}
		
		// Rinfresca la barra delle primitive
		if( !mainframe->m_bDestroyCacheThread)
			mainframe->LoadMacros();

		// Riarma il meccanismo di notifica
		do
		{
			::FindNextChangeNotification( hNotif);
		}
		while( ::WaitForSingleObject( hNotif, 100) == WAIT_OBJECT_0);
	}

	::FindCloseChangeNotification( hNotif);
	mainframe->m_CacheUpdateThread = NULL;
	return 0;
}

void CMainFrame::OnViewLayers() 
{
	CDlgLayers dlg;
	if( dlg.DoModal() == IDOK)
	{
		/*
			Devo scorrere tutti i documenti aperti per fare due cose: uno, nascondere
			i componenti che ora sono nei layer invisibili e viceversa; due,
			deselezionare tutti i componenti invisibili dal SelArray.
		*/

		POSITION pos = m_DocCache.GetHeadPosition();
		while( pos != NULL)
		{
			CElecDrawDoc *d = (CElecDrawDoc*)m_DocCache.GetNext( pos);

			// Tolgo dal SelArray tutti i componenti che sono diventati invisibili			
			PRIMARRAY& SelArray = d->m_selArray;
			int a, n = SelArray.GetSize();
			CPrimitive *p;
			for( a=n-1; a>=0; a--)
			{
				p = SelArray.GetAt(a);
				if( !CElecDrawView::IsComponentVisible(p))
				{
					p->Select( FALSE);
					SelArray.RemoveAt(a);
				}
			}
			
			// Aggiorno tutte le viste del documento			
			d->UpdateAllViews( NULL);
		}

		// Devo inoltre reimpostare la combo dei layer, dato che nomi/colori possono essere cambiati
		int lsel = m_comboLayer.GetCurSel();
		m_comboLayer.ResetContent();
		int a, id;
		for( a=0; a<16; a++)
		{
			id = m_comboLayer.AddString( m_layerNames[a]);
			m_comboLayer.SetItemData( id, a+1);
		}
		m_comboLayer.SelectString( -1, m_layerNames[lsel]);
	}
}

void CMainFrame::OnLayerChange()
{
	int idx = m_comboLayer.GetCurSel();
	if( idx != CB_ERR)
	{
		m_currentLayer = m_comboLayer.GetItemData( idx)-1;

		/* Se ci sono componenti selezioniati nella finestra attiva, gli cambiamo il layer */
		CFrameWnd *pChild = GetActiveFrame();
		CElecDrawView *pView = (CElecDrawView *) pChild->GetActiveView();

		int a, n;
		if( (pView != NULL) && (n = pView->GetDocument()->m_selArray.GetSize()) != 0)
		{
			CPrimitive *p;
			for( a=0; a<n; a++)
			{
				p = pView->GetDocument()->m_selArray[a];
				p->SetLayer( m_currentLayer);
				p->Select( FALSE);
			}

			/* Dopo deselezioniamo tutto e aggiorniamo */
			pView->GetDocument()->m_selArray.RemoveAll();
			pView->Invalidate();
			pView->UpdateWindow();

			/* Inoltre memorizziamo nell'undo buffer */
			pView->GetDocument()->UndoStore();
		}
	}
}

void CMainFrame::LoadLibNames()
{
	CString key( "LibNames");

	_FileNamesCache.RemoveAll();
	int a, n = AfxGetApp()->GetProfileInt( key, "n", 0);
	CString str, value;
	for( a=0; a<n; a++)
	{
		value.Format( "name%d", a);
		str = AfxGetApp()->GetProfileString( key, value, "");
		if( str != "")
			_FileNamesCache.SetAt( str, str);
	}
}

void CMainFrame::SaveLibNames()
{
	CString key( "LibNames");

	AfxGetApp()->WriteProfileInt( key, "n", _FileNamesCache.GetCount());

	POSITION pos = _FileNamesCache.GetStartPosition();
	int n=0;
	CString value, s1, s2;
	while( pos)
	{
		_FileNamesCache.GetNextAssoc( pos, s1, s2);
		value.Format( "name%d", n++);
		AfxGetApp()->WriteProfileString( key, value, s1);
	}
}

/*void CMainFrame::OnOptionsLibbuf() 
{
	CDlgLibBuf dlg;
	if( dlg.DoModal() == IDOK)
		SaveLibNames();
}
*/
