/*
	PRIMITIVE.CPP
	Definizione delle primitive di disegno

	Leggere i commenti che si trovano fra una funzione e l'altra
*/


#include "stdafx.h"
#include "ElecDraw.h"
#include "Primitive.h"
#include "DlgMacroBar.h"
#include "ConfigDialog.h"
#include "GridDialog.h"
#include "ViewToolbar.h"
#include "ElecDrawDoc.h"
#include "ElecDrawView.h"
#include "LayerComboBox.h"
#include "MainFrm.h"

#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_SERIAL( CLine, CObject, 1)
IMPLEMENT_SERIAL( CExtendedText, CObject, 1)
IMPLEMENT_SERIAL( CConnection, CObject, 1)
IMPLEMENT_SERIAL( CBezier, CObject, 1)
IMPLEMENT_SERIAL( CPoly, CObject, 1)
IMPLEMENT_SERIAL( CEllipse, CObject, 1)
IMPLEMENT_SERIAL( CRectangle, CObject, 1)
IMPLEMENT_SERIAL( CMacro, CObject, 1)
IMPLEMENT_SERIAL( CPCBLine, CObject, 1)
IMPLEMENT_SERIAL( CPCBPad, CObject, 1)


/*
	Mh? Come dite? Le variabili globali non si devono usare? Andatelo a dire
	a chi gliene frega qualcosa :)
*/

static CExtendedText	_lastText;
static CRectangle		_lastRect;
static CEllipse			_lastEllipse;
static CPoly			_lastPoly;
static CPCBLine			_lastVia;
static CPCBPad			_lastPad;


///////////////////////////////////////////////////////////////////////////////////
// CPrimitive


/*
	Considerazioni su directory & estensioni

	In un primo momento avevo predisposto il codice affinche' potesse accettare una
	directory e/o un'estensione alternative per la ricerca/caricamento delle macro. Con
	le librerie la situazione si sarebbe complicata ulteriormente, quindi ecco la mia
	scelta: ho eliminato la possibilita' di passare tali argomenti alle funzioni di
	manipolazione delle macro, ripulendo e semplificando moltissimo il codice a
	discapito di una funzionalita' che non e' fondamentale. Ora le directory e le
	estensioni sono rigide come definite nelle costanti in ElecDraw.h, MACROS_X e
	LIBS_X.

	Nonostante cio', e' possibile che in futuro ci sia la necessita' di
	supportare eventuali directory/estensioni aggiuntive. In tal caso, l'unica funzione
	helper di CPrimitive che deve essere modificata per riflettere le modifiche e' 
	
	CPrimitive::ScanMacroName
	
	Tutte le altre funzioni (inclusa CMacro::Load) si adegueranno automagicamente.
	Ovviamente le modifiche al resto del codice sono un'altra cosa... :-)
*/


/*
	CleanBuffer (statico)
	Ripulisce il buffer, adattandolo a FidoCAD.
	In particolare:
	1) Converte tutti i ritorni a capo nella forma di Windows
	2) Strippa tutti i caratteri invalidi
*/

void CPrimitive::CleanBuffer( CString& _buffer)
{
	int a, b=0, n=_buffer.GetLength();
	const char *buffer = LPCSTR(_buffer);
	char *pulito = new char[n];

	for( a=0; a<n; a++)
	{
		if( buffer[a] == 13 || buffer[a] == 10)
		{
			pulito[b++] = '\n';
			if( a<n-1 && (buffer[a+1] == 13 || buffer[a+1] == 10))
				a++;
			continue;
		}

		if( buffer[a] < 32 || buffer[a] > 127)
			pulito[b++] = ' ';
		else
			pulito[b++] = buffer[a];
	}

	buffer = CString(pulito);
	delete[] pulito;
}


/*
	ExtractString (statico)
	Estrae la stringa di testo da un comando FidoCAD

	Parametri:
->	linea: parametri del comando
->	nOggettiPrima: numero di parametri prima della stringa

	Restituisce:
	La stringa desiderata, o "" in caso di problemi.
*/

CString CPrimitive::ExtractString( CString& linea, int nOggettiPrima)
{
	int n;
	int i=0, max=linea.GetLength();

	while( (unsigned char)(linea[i]) <= ' ' && i<max)
		i++;

	for( n=0; n<nOggettiPrima; n++)
	{
		while( (unsigned char)(linea[i]) > ' ' && i<max)
			i++;

		while( (unsigned char)(linea[i]) <= ' ' && i<max)
			i++;
	}
	
	CString str("");
	if( i<max)
	{
		str = linea.Mid( i);
		
		// Secca eventuali skifezze in fondo alla stringa
		int b, m=str.GetLength();
		for( b=0; b<m; b++)
			if( (unsigned char)(str[b]) < ' ')
			{
				str = str.Left( b+1);
				break;
			}
	}

	return str;
}


/*
	ScanLine (statico)
	Converte una linea di comando FidoCAD nella primitiva corrispondente.

	Parametri:
->	str: stringa che contiene un singolo comando.

	Restituisce:
	La nuova primitiva allocata nello heap in caso di successo, NULL altrimenti.
*/

CPrimitive* CPrimitive::ScanLine( CString& str)
{
	CPrimitive* prim;

	if( (prim = CLine::Load( str)) != NULL)
		return prim;
	else if( (prim = CExtendedText::Load( str)) != NULL)
		return prim;
	else if( (prim = CConnection::Load( str)) != NULL)
		return prim;
	else if( (prim = CBezier::Load( str)) != NULL)
		return prim;
	else if( (prim = CPoly::Load( str)) != NULL)
		return prim;
	else if( (prim = CEllipse::Load( str)) != NULL)
		return prim;
	else if( (prim = CRectangle::Load( str)) != NULL)
		return prim;
	else if( (prim = CMacro::Load( str)) != NULL)
		return prim;
	else if( (prim = CPCBLine::Load( str)) != NULL)
		return prim;
	else if( (prim = CPCBPad::Load( str)) != NULL)
		return prim;

	return NULL;
}


/*
	ScanBuffer (statico)
	Converte un buffer di righe di comando in un array delle corrispondenti primitive.
	Non e' adatto per fare lo scan del corpo di una libreria; per quest'ultima
	operazione usare ScanLibraryBuffer.

	Parametri:
->	str: stringa che contiene il corpo di un documento (senza intestazione).
<-	ar: array che conterra' le primitive scannate. Se l'array contiene degli elementi,
	     esso verra' svuotato e gli elementi cancellati.
<-	bWarning: se questo parametro non e' NULL, eventuali errori nell'interpretazione
	          di uno o piu' comandi verranno evidenziati ponendo il flag a TRUE. Da
		      notare che tali errori non sono critici e non interrompono il lavoro
			  della funzione.

	Restituisce:
	TRUE in caso di successo, FALSE altrimenti.
*/

BOOL CPrimitive::ScanBuffer( CString& _str, PRIMARRAY& ar, BOOL *bWarning)
{
	// Resetta il flag di warning
	if( bWarning != NULL)
		*bWarning = FALSE;

	// Svuota l'obj array
	int b, m=ar.GetSize();
	for( b=0; b<m; b++)
		delete (ar.GetAt(b));
	ar.RemoveAll();

	// Scanna le linee (non si aspetta nessun header all'inizio!)
	const char* str = LPCSTR(_str);
	int a, n=_str.GetLength(), start = 0;
	for( a=0; a<n; a++)
	{
		if( str[a] == '\n' || a==n-1)
		{
			CString substr;
			if( a==n-1)
				substr = _str.Mid( start, a-start+1);
			else
				substr = _str.Mid( start, a-start);

			substr.TrimLeft();
			substr.TrimRight();
			if( substr != "")
			{				
				CPrimitive *prim = CPrimitive::ScanLine( substr);
				if( prim == NULL)
				{
					if( bWarning != NULL)
						*bWarning = TRUE;
				}
				else
					ar.Add( prim);
			}

			start = a+1;
		}
	}

	return TRUE;
}


/*
	ScanClipboard (statico)
	Legge il contenuto della clipboard

	Parametri:
<-	&ar: array in cui verranno posti gli oggetti scannati dalla clipboard.
->	MainWnd: puntatore alla finestra principale dell'applicazione. Tipicamente
	         si legge con AfxGetMainWnd(). Non puo' essere NULL.
<-	bWarning: vedi ScanBuffer.

	Restituisce:
	TRUE in caso di successo, FALSE altrimenti.
*/

BOOL CPrimitive::ScanClipboard( PRIMARRAY& ar, CWnd* MainWnd, BOOL* bWarning)
{
	if( MainWnd == NULL)
		return FALSE;

	if( bWarning != NULL)
		*bWarning = FALSE;

	// Apre la clipboard
	if( !MainWnd->OpenClipboard())
		return FALSE;

	// Legge i dati
	HANDLE h;
	void *p;
	if( (h = ::GetClipboardData( CF_TEXT)) == NULL ||
		(p = ::GlobalLock( h)) == NULL)
			return FALSE;
	
	// Converte i dati in stringa
	CString clip( (char *)p);

	::GlobalUnlock( h);
	::CloseClipboard();

	// Legge le primitive
	return CPrimitive::ScanBuffer( clip, ar, bWarning);
}

DOC_TYPE CPrimitive::GetType( CString& buf1)
{
	// TODO: codice duplicato

	CString buffer = buf1;

	// Ripulisce il buffer
	CPrimitive::CleanBuffer( buffer);

	// Va in cerca dell'header (tutto quello che c'e' prima e` ignorato)
	int n=0, m=-1, d=-1, l=-1, sz=buffer.GetLength();
	int lungh=0;
	while( n < sz)
	{
		while( n<sz-1 && buffer[n] != '[') n++;
		if( n == sz-1)
			return DOC_NONE;

		CString stt = buffer.Mid(n+1);
		stt.TrimLeft();
		if( stt.Left(lstrlen(DOCS_HEADER)) == DOCS_HEADER)
		{
			d = n;
			lungh = lstrlen(DOCS_HEADER);
			break;
		}
		if( stt.Left(lstrlen(MACROS_HEADER)) == MACROS_HEADER)
		{
			m = n;
			lungh = lstrlen(MACROS_HEADER);
			break;
		}
		if( stt.Left(lstrlen(LIBS_HEADER)) == LIBS_HEADER)
		{
			l = n;
			lungh = lstrlen(LIBS_HEADER);
			break;
		}

		n++;
	}

	if( d == -1 && m == -1 && l == -1)
		return DOC_NONE;
	else if( d != -1)
		return DOC_NORMAL;
	else if( m != -1)
		return DOC_MACRO;
	else
		return DOC_LIBRARY;
}


/*
	ScanHeadedBuffer
	Esamina un buffer che puo' contenere un documento di qualsiasi tipo, incluso
	l'header. Tutto cio' che c'e' prima dell'header viene ignorato.

	Parametri:
->	str: stringa che contiene il buffer.
<-	ar: puntatore al PRIMARRAY che ricevera' gli oggetti scannati (se il buffer
	    contiene un documento normale o una macro). Eventuali oggetti contenuti in
		*ar verranno distrutti. Se il parametro e' NULL non verra' effettuato lo scan
		di tali oggetti (ma solo dell'header). Quando il buffer contiene una libreria,
		questo parametro viene ignorato.
<-	lm: puntatore a un STRARRAY che ricevera' i blocchi-macro scannati (se il
	    buffer contiene una libreria). Eventuali stringhe contenute in *lm verranno
		distrutte. Se il parametro e' NULL non verra' effettuato lo scan di tali
		blocchi (ma solo dell'header). Quando il buffer contiene un documento normale
		o una macro, questo parametro viene ignorato.
<-	title: puntatore a una stringa che accettera' il titolo lungo del documento.
           Il parametro puo' essere NULL.
<-	bWarning: vedi ScanBuffer.
<-	t: puntatore a un DOC_TYPE che accettera' il tipo di documento contenuto nel
       buffer. Il parametro puo' essere NULL.

	Restuituisce:
	TRUE in caso di successo, FALSE altrimenti. L'assenza (o formato invalido)
	dell'header costituisce un errore critico.
*/

BOOL CPrimitive::ScanHeadedBuffer( CString& buffer, PRIMARRAY* ar,
	STRARRAY* lm, CString *title, BOOL *bWarning, DOC_TYPE *t, SUBDIRARRAY* sda)
{
	if( bWarning != NULL)
		*bWarning = FALSE;

	// Ripulisce il buffer
	CPrimitive::CleanBuffer( buffer);

	// Va in cerca dell'header (tutto quello che c'e' prima e` ignorato)
	int n=0, m=-1, d=-1, l=-1, sz=buffer.GetLength();
	int lungh=0;
	while( n < sz)
	{
		while( n<sz-1 && buffer[n] != '[') n++;
		if( n == sz-1)
			return FALSE;

		CString stt = buffer.Mid(n+1);
		stt.TrimLeft();
		if( stt.Left(lstrlen(DOCS_HEADER)) == DOCS_HEADER)
		{
			d = n;
			lungh = lstrlen(DOCS_HEADER);
			break;
		}
		if( stt.Left(lstrlen(MACROS_HEADER)) == MACROS_HEADER)
		{
			m = n;
			lungh = lstrlen(MACROS_HEADER);
			break;
		}
		if( stt.Left(lstrlen(LIBS_HEADER)) == LIBS_HEADER)
		{
			l = n;
			lungh = lstrlen(LIBS_HEADER);
			break;
		}

		n++;
	}

	if( d == -1 && m == -1 && l == -1)
		return FALSE;

	// Imposta il flag macro
	DOC_TYPE tt;
	if( d != -1)
		tt = DOC_NORMAL;
	else if( m != -1)
		tt = DOC_MACRO;
	else
		tt = DOC_LIBRARY;

	if( t != NULL)
		*t = tt;

	int st = (d!=-1) ? d : (m!=-1) ? m : l;

	// Trova la fine dell'header
	CString str1 = buffer.Mid( st+1);
	str1.TrimLeft();
	str1 = str1.Mid( lungh);
	str1.TrimLeft();

	// Imposta il titolo
	int stop = str1.Find(']');
	int newline = str1.Find('\n');
	if( stop == -1 || (newline != -1 && stop > newline))
		return FALSE;

	if( title != NULL)
		*title = str1.Left( stop);

	CString substr = str1.Mid( stop+1);

	// Discriminazione razziale tra tipi di file
	if( tt == DOC_LIBRARY)
	{
		if( lm == NULL)
			return TRUE;

		return CMacro::ScanLibraryBuffer( substr, *lm, sda);
	}
	else
	{
		if( ar == NULL)
			return TRUE;
		
		return CPrimitive::ScanBuffer( substr, *ar, bWarning);
	}
}


/*
	ScanFile (statico)
	Come ScanHeadedBuffer, ma e' lui stesso che pensa a caricare il buffer dato il
	nome del file (filename).
*/

BOOL CPrimitive::ScanFile( CString& filename, PRIMARRAY* ar, STRARRAY* lm,
	CString *title, BOOL *bWarning, DOC_TYPE *t, CString* ExternalBuffer)
{
	if( bWarning != NULL)
		*bWarning = FALSE;

	CString buffer("");
	try
	{
		// Legge il file
		CFile file;
		if( !file.Open( filename, CFile::modeRead | CFile::shareExclusive))
			return FALSE;

		char minibuf[1000];
		UINT nRead;
		do
		{
			nRead = file.Read( minibuf, 1000);
			buffer += CString( minibuf, nRead);
		} while( nRead == 1000);

		file.Close();
	}
	catch( CFileException *e)
	{
		delete e;
		return FALSE;
	}

	if( ExternalBuffer != NULL)
	{
		*ExternalBuffer = buffer;
		return TRUE;
	}
	else
		return CPrimitive::ScanHeadedBuffer( buffer, ar, lm, title, bWarning, t);
}



/***********************************************************************************/

CPrimitive::CPrimitive()
{
	m_bSelected = FALSE;
	m_nRotations = m_nSavedRotations = 0;
	m_bSwapped = m_bSavedSwapped = FALSE;

	m_bConst = FALSE;
	m_nLayer = 0;

	for( int a=0; a<MAX_DEFINING_POINTS+1; a++)
		m_ptDefining[a] = m_ptSaved[a] = CPoint(-1,-1);
}

CPrimitive::~CPrimitive()
{
}

CPrimitive* CPrimitive::Clone()
{
	CPrimitive* p = new CPrimitive();
	*p = *(this);
	return p;
}

BOOL CPrimitive::operator= (CPrimitive& src)
{
	// Copia tutto e amorevolmente (anche il backup, of cooooz)
	int a;
	for( a=0; a<MAX_DEFINING_POINTS+1; a++)
	{
		m_ptDefining[a] = src.m_ptDefining[a];
		m_ptSaved[a] = src.m_ptSaved[a];
	}

	m_bSelected = src.m_bSelected;
	m_nRotations = src.m_nRotations;
	m_bSwapped = src.m_bSwapped;
	m_bConst = src.m_bConst;
	m_nSavedRotations = src.m_nSavedRotations;
	m_bSavedSwapped = src.m_bSavedSwapped;
	m_nLayer = src.m_nLayer;

	/*
		TODO: commentare il codice seguente evita un sacco di crash e problemi, e
		sembra che non influisca con nessuna funzione del programma. E` comunque
		necessario indagare bene sulla cosa e capire se c'e` qualcosa che
		non funziona piu`.
	*/

/*	// Crea una nuova regione, se non esiste
	if( m_Region.m_hObject == NULL)
		m_Region.CreateRectRgn( 0, 0, 1, 1);

	// Copia la nuova regione (se la nuova esiste)
	if( m_Region.m_hObject != NULL && src.m_Region.m_hObject != NULL)
		m_Region.CopyRgn( &(src.m_Region));
*/
	return TRUE;
}

/*
	MoveRel
	Muove la primitiva (shadow point compreso) di una distanza relativa.
*/
void CPrimitive::MoveRel( CPoint ptrel)
{
	int a, n=GetNumDefPoints()+1;

	for( a=0; a<n; a++)
	{
		m_ptDefining[a].x += ptrel.x;
		m_ptDefining[a].y += ptrel.y;
	}
}

/*
	Rotate
	Ruota la primitiva (shadow point compreso) in senso orario o antiorario rispetto
	a un punto. Tale punto puo' coincidere o meno con uno dei punti di definizione.
*/
void CPrimitive::Rotate( CPoint pt, BOOL bCounterClockWise)
{
	int b, m=GetNumDefPoints()+1;
	CPoint ptTmp;

	if( !(GetProp() & (PROP_ROTATEFULL | PROP_ROTATEHALF)))
		return;

	for( b=0; b<m; b++)
	{
		ptTmp = m_ptDefining[b];

		if( !bCounterClockWise)
		{
			m_ptDefining[b].x = pt.x - (ptTmp.y-pt.y);
			m_ptDefining[b].y = pt.y + (ptTmp.x-pt.x);
		}
		else
		{
			m_ptDefining[b].x = pt.x + (ptTmp.y-pt.y);
			m_ptDefining[b].y = pt.y - (ptTmp.x-pt.x);
		}
	}
}

/*
	Swap
	Trasforma la primitiva (shadow point compreso) nel suo speculare rispetto a 
	un asse verticale. L'asse di simmetria e' la retta verticale x=xPos.
*/
void CPrimitive::Swap( int xPos)
{
	int a, n=GetNumDefPoints()+1;
	int xtmp;

	if( !(GetProp() & (PROP_SWAPFULL | PROP_SWAPHALF)))
		return;

	for( a=0; a<n; a++)
	{
		xtmp = m_ptDefining[a].x;			// Opzione +LuziCompiler
		m_ptDefining[a].x = 2*xPos - xtmp;
	}
}

/*
	UpdateBoundingRegion
	Aggiorna la regione che circonda la primitiva. Se lo facevo automaticamente
	a ogni modifica sarebbe stato troppo lento, cosi' e' il programma che deve
	intelligentemente aggiornare le cose quando gli servono.
*/
void CPrimitive::UpdateBoundingRgn( CDC *pdc)
{
	CDC memdc;

	memdc.CreateCompatibleDC( pdc);
	memdc.BeginPath();
	Draw( &memdc, FALSE);
	memdc.CloseFigure();
	memdc.EndPath();

	// Elimina la vecchia regione associata
	HRGN rg;
	rg = (HRGN)(m_Region.Detach());
	if( rg != NULL)
		::DeleteObject( rg);

	// Crea e attacca la nuova regione
	rg = ::PathToRegion( memdc.m_hDC);
	m_Region.Attach( rg);
}

/*
	IsPtNear
	Verifica se il punto specificato e' dentro alla primitiva, e se c'e' se e'
	sul corpo o su uno dei punti di definizione.
	NOTA: il punto deve essere passato in COORDINATE FISICHE, in modo da rendere piu'
	agevole il check sui punti di definizione (sul corpo non c'e' niente da fare, la
	regione DEVE essere in coord. logiche).
*/
PROXIMITY_TYPE CPrimitive::IsPtNear( CPoint ptRaw, CDC *pdc)
{
	int a, n=GetNumDefPoints();

	if( m_bConst)
		return NEAR_NONE;

	// Vicinanza a un punto di definizione (anche nei paraggi)
	CPoint ptDefDev;
	for( a=0; a<n; a++)
	{
		ptDefDev = GetPoint(a);
		pdc->LPtoDP( &ptDefDev);
		if( ABS(ptRaw.x-ptDefDev.x) < SELECTOR_SIZE/2 &&
			ABS(ptRaw.y-ptDefDev.y) < SELECTOR_SIZE/2)
				return (PROXIMITY_TYPE)(NEAR_SELECTOR_0+a);
	}

	// Vicinanza al corpo
	pdc->DPtoLP( &ptRaw);
	if( m_Region.m_hObject != NULL && m_Region.PtInRegion( ptRaw))
		return NEAR_BODY;

	return NEAR_NONE;
}

/*
	IsProper
	Fa un controllo di correttezza della primitiva.
*/
BOOL CPrimitive::IsProper()
{
	int a, n=GetNumDefPoints();

	// Per ora il controllo si limita a verificare che i punti di definizione
	// siano nell'area di disegno
	for( a=0; a<n; a++)
		if( m_ptDefining[a].x == -1 && m_ptDefining[a].y == -1)
				return FALSE;

	return TRUE;
}

/*
	SaveStates e LoadStates
	Salva e ricarica i dati salienti di una primitiva. Per ora non tutto viene salvato,
	visto che queste due funzioni sono utilizzate soltanto dall'editor in fase di
	trascinamento/modifica (in tale fase possono variare solo i punti di definizione
	e il numero di rotazioni).
*/
void CPrimitive::SaveState( void)
{
	int a, n=GetNumDefPoints()+1;
	for( a=0; a<n; a++)
		m_ptSaved[a] = m_ptDefining[a];

	m_nSavedRotations = m_nRotations;
	m_bSavedSwapped = m_bSwapped;
}

void CPrimitive::LoadState( void)
{
	int a, n=GetNumDefPoints()+1;
	for( a=0; a<n; a++)
		m_ptDefining[a] = m_ptSaved[a];

	m_nRotations = m_nSavedRotations;
	m_bSwapped = m_bSavedSwapped;
}

/*
	GetPoint e SetPoint
	Impostano e leggono uno dei punti di definizione. Lo shadow point non e' ne'
	modificabile ne' impostabile.
*/
CPoint CPrimitive::GetPoint( int nIndex)
{
	if( nIndex < GetNumDefPoints())
		return m_ptDefining[nIndex];
	else
		return CPoint(-1,-1);
}

void CPrimitive::SetPoint( int nIndex, CPoint pt, BOOL bSignal)
{
	if( nIndex < GetNumDefPoints())
		m_ptDefining[nIndex] = pt;
}

/*
	Select e IsSelected
	Impostano/restituiscono lo stato di selezione dell'oggetto.
*/
void CPrimitive::Select( BOOL bSel)
{
	m_bSelected = bSel;
}

BOOL CPrimitive::IsSelected( void)
{
	return m_bSelected;
}

/*
	SetConst e IsConst
	Impostano/restituiscono lo stato di "costanza" dell'oggetto (se esso e'
	una costante, e non si puo' modificare).
*/
void CPrimitive::SetConst( BOOL bConst)
{
	m_bConst = bConst;
}

BOOL CPrimitive::IsConst( void)
{
	return m_bConst;
}


/*
	GetDrawPriority
	Restituisce la "priorita` di disegno" della primitiva. Una primitiva con priorita`
	minore viene disegnata *prima*, quindi viene sovrascritta dalle altre primitive.
	Le priorita` vanno da 0 a 6; il default e` 3, valore intermedio.
*/
int CPrimitive::GetDrawPriority( void)
{
	return 3;
}


/*
	GetRegion
	Restituisce un puntatore alla regione della primitiva.
*/
CRgn* CPrimitive::GetRegion( void)
{
	return &m_Region;
}


/*
	RegioneLinea (statico)
	Imposta la CRgn passata in modo che essa rappresenti la regione attorno alla
	linea passata, con una determinata larghezza (indipendente dalla scala)
*/
BOOL CPrimitive::RegioneLinea( CDC* pdc, CRgn& rgn, CPoint pt1, CPoint pt2, int largh)
{
	CSize l( largh, largh);
	pdc->DPtoLP( &l);
	int maxlargh = (l.cx > l.cy) ? l.cx : l.cy;
	if( maxlargh < 2)
		maxlargh = 2;

	CPoint pts[4];
	if( pt2.y > pt1.y && pt2.x > pt1.x ||
		pt2.y < pt1.y && pt2.x < pt1.x)
	{
		pts[0].x = pt1.x - maxlargh/2;
		pts[0].y = pt1.y + maxlargh/2;
		pts[1].x = pt2.x - maxlargh/2;
		pts[1].y = pt2.y + maxlargh/2;
		pts[2].x = pt2.x + maxlargh/2;
		pts[2].y = pt2.y - maxlargh/2;
		pts[3].x = pt1.x + maxlargh/2;
		pts[3].y = pt1.y - maxlargh/2;
	}
	else
	{
		pts[0].x = pt1.x + maxlargh/2;
		pts[0].y = pt1.y + maxlargh/2;
		pts[1].x = pt2.x + maxlargh/2;
		pts[1].y = pt2.y + maxlargh/2;
		pts[2].x = pt2.x - maxlargh/2;
		pts[2].y = pt2.y - maxlargh/2;
		pts[3].x = pt1.x - maxlargh/2;
		pts[3].y = pt1.y - maxlargh/2;
	}

	// Elimina la vecchia regione associata
	HRGN rg;
	rg = (HRGN)(rgn.Detach());
	if( rg != NULL)
		::DeleteObject( rg);

	// Crea e attacca la nuova regione
	rgn.CreatePolygonRgn( pts, 4, WINDING);
	return TRUE;
}

void CPrimitive::Rimpiazza( CString& str, const char *src, const char *dest)
{
	CString tmp = str;
	int l = lstrlen( src);

	str = "";
	while( tmp != "")
	{
		if( tmp.Left( l) == src)
		{
			str += dest;
			tmp = tmp.Mid( l);
		}
		else
		{
			str += tmp[0];
			tmp = tmp.Mid( 1);
		}
	}
}

void CPrimitive::InitLastSettings()
{
	_lastText.m_Angle = 0;
	_lastText.m_bConst = FALSE;
	_lastText.m_Flags = 0;
	_lastText.m_FontFace = "Courier New";
	_lastText.m_FontHeight = 5;
	_lastText.m_FontWidth = 3;

	_lastRect.m_bConst = FALSE;
	_lastRect.m_bFilled = FALSE;
	_lastEllipse.m_bConst = FALSE;
	_lastEllipse.m_bFilled = FALSE;
	_lastPoly.m_bConst = FALSE;;
	_lastPoly.m_bFilled = FALSE;

	_lastVia.m_bConst = FALSE;
	_lastVia.m_thickness = 4;

	_lastPad.m_bConst = FALSE;
	_lastPad.m_intdiam = 4;
	_lastPad.m_sizex = 18;
	_lastPad.m_sizey = 18;
	_lastPad.m_style = CPCBPad::PAD_ROUND;
}

///////////////////////////////////////////////////////////////////////////////////
// CLine
///////////////////////////////////////////////////////////////////////////////////

CLine::CLine()
{
}

CLine::CLine( CPoint start, CPoint stop, BOOL bConst)
{
	m_ptDefining[0] = m_ptSaved[0] = start;
	m_ptDefining[1] = m_ptSaved[1] = stop;
	m_bConst = bConst;
}

CLine::~CLine()
{
}

CPrimitive* CLine::Clone()
{
	CLine* p = new CLine();
	*p = *(this);
	return p;
}

void CLine::Draw( CDC *pdc, BOOL bXNOR, COLORREF backcolor, COLORREF forecolor, COLORREF selcolor, int zoom, int nLayer, BOOL blackandwhite)
{
	// Se c'e' un punto che vale (-1,-1), qualcuno ha fatto dei poltigli e
	// quindi non disegno nulla
	if( m_ptDefining[0].x == -1 && m_ptDefining[0].y == -1 ||
		m_ptDefining[1].x == -1 && m_ptDefining[1].y == -1)
		return;

	CPen *oldpen, pen( PS_SOLID, 0, m_bSelected && !bXNOR ? selcolor : forecolor);
	int oldROP = pdc->m_hAttribDC == NULL ? R2_COPYPEN : pdc->GetROP2();

	if( bXNOR)
		pdc->SetROP2( R2_NOTXORPEN);
	else
		pdc->SetROP2( R2_COPYPEN);

	oldpen = pdc->SelectObject( &pen);

	pdc->MoveTo( m_ptDefining[0].x*zoom/100, m_ptDefining[0].y*zoom/100);
	pdc->LineTo( m_ptDefining[1].x*zoom/100, m_ptDefining[1].y*zoom/100);
	pdc->SetPixel( m_ptDefining[1].x*zoom/100, m_ptDefining[1].y*zoom/100, forecolor);

	pdc->SelectObject( oldpen);
	pen.DeleteObject();
	pdc->SetROP2( oldROP);
}

void CLine::UpdateBoundingRgn( CDC *pdc)
{
	CPrimitive::RegioneLinea( pdc, m_Region, m_ptDefining[0], m_ptDefining[1], SELECTOR_SIZE);
}

int CLine::GetNumDefPoints( void)
{
	return 2;
}

DWORD CLine::GetProp( void)
{
	return PROP_ROTATEFULL | PROP_SWAPFULL | PROP_UPDATEONZOOM | PROP_SERIALCREATE;
}

CPrimitive* CLine::Load( CString& linea)
{
	linea.TrimLeft();
	CString cmp = linea.Left(2);
	cmp.MakeUpper();

	if( cmp != "LI")
		return NULL;
	
	CLine* prim = new CLine();
	CString buf = linea.Right( linea.GetLength()-2);
	prim->m_nLayer = 0;
	sscanf( LPCSTR(buf), "%d %d %d %d %d", &(prim->m_ptDefining[0].x),
		&(prim->m_ptDefining[0].y), &(prim->m_ptDefining[1].x),
		&(prim->m_ptDefining[1].y), &(prim->m_nLayer));

	if( !prim->IsProper())
	{
		delete prim;
		return NULL;
	}
	
	prim->SaveState();
	return prim;
}

BOOL CLine::Save( CString& str, BOOL bSplit)
{
	str.Format( "LI %d %d %d %d", m_ptDefining[0].x, m_ptDefining[0].y,
		m_ptDefining[1].x, m_ptDefining[1].y);

	if( m_nLayer == 0)
		str += "\r\n";
	else
	{
		CString l;
		l.Format( " %d\r\n", m_nLayer);
		str += l;
	}

	return TRUE;
}

void CLine::Serialize( CArchive &ar)
{
	if( ar.IsStoring())
	{
		ar <<	m_ptDefining[0] << m_ptDefining[1];
	}
	else
	{
		ar >>	m_ptDefining[0] >> m_ptDefining[1];
	}
}


///////////////////////////////////////////////////////////////////////////////////
// CExtendedText
///////////////////////////////////////////////////////////////////////////////////

CExtendedText::CExtendedText()
{
}

CExtendedText::CExtendedText( CPoint start, CString text, CString face, int height,
			int width, int flags, int angle, BOOL bConst)
{
	// Pesca ultimi settaggi dal pool
	*this = _lastText;

	// Alcuni attributi sono sempre da cambiare
	m_ptDefining[0] = m_ptSaved[0] = start;
	m_sText = text;

	RecreateFont();
}

BOOL CExtendedText::RecreateFont( int zoom)
{
	m_Font.DeleteObject();
	if( m_Font.CreateFont( m_FontHeight*zoom/100, m_FontWidth*zoom/100,
		m_Angle*10, 0,
		m_Flags & EXTFONT_STYLE_BOLD ? FW_BOLD : FW_NORMAL,
		m_Flags & EXTFONT_STYLE_ITALIC ? TRUE : FALSE,
		m_Flags & EXTFONT_STYLE_UNDERLINED ? TRUE : FALSE,
		FALSE,
		ANSI_CHARSET,
		OUT_TT_PRECIS,
		CLIP_TT_ALWAYS | CLIP_LH_ANGLES,
		DEFAULT_QUALITY,
		FIXED_PITCH | TMPF_TRUETYPE,
		m_FontFace) == FALSE)
	{
		return m_Font.CreateFont( m_FontHeight*zoom/100, m_FontWidth*zoom/100,
		m_Angle*10, 0,
		m_Flags & EXTFONT_STYLE_BOLD ? FW_BOLD : FW_NORMAL,
		m_Flags & EXTFONT_STYLE_ITALIC ? TRUE : FALSE,
		m_Flags & EXTFONT_STYLE_UNDERLINED ? TRUE : FALSE,
		FALSE,
		ANSI_CHARSET,
		OUT_TT_PRECIS,
		CLIP_TT_ALWAYS | CLIP_LH_ANGLES,
		DEFAULT_QUALITY,
		FIXED_PITCH | TMPF_TRUETYPE,
		"Courier New");
	}
	else
		return TRUE;
}

void CExtendedText::GetBoundingPoly( POINT *pts, CSize& s, int zoom, BOOL bNoSwap)
{
	double cosx = cos( (double)m_Angle/360*2*3.1416);
	double senx = sin( (double)m_Angle/360*2*3.1416);
	pts[0].x = m_ptDefining[0].x*zoom/100;
	pts[0].y = m_ptDefining[0].y*zoom/100;
	pts[1].x = pts[0].x + (int)(s.cy * senx);
	pts[1].y = pts[0].y + (int)(s.cy * cosx);
	pts[2].x = pts[1].x + (int)(s.cx * cosx);
	pts[2].y = pts[1].y - (int)(s.cx * senx);
	pts[3].x = pts[0].x + (int)(s.cx * cosx);
	pts[3].y = pts[0].y - (int)(s.cx * senx);

	if( !bNoSwap && (m_Flags & EXTFONT_STYLE_MIRRORED))
	{
		int a, xtmp;
		for( a=1; a<4; a++)
		{
			xtmp = pts[a].x;
			pts[a].x = 2*pts[0].x - xtmp;
		}
	}
}

CRect CExtendedText::GetBoundingPolyRect( POINT *pts)
{
	CRect rc( 100000, 100000, -100000, -100000);
	int a;

	for( a=0; a<4; a++)
	{
		if( pts[a].x < rc.left)
			rc.left = pts[a].x;
		if( pts[a].x > rc.right)
			rc.right = pts[a].x;
		if( pts[a].y < rc.top)
			rc.top = pts[a].y;
		if( pts[a].y > rc.bottom)
			rc.bottom = pts[a].y;
	}

	return rc;
}

CExtendedText::~CExtendedText()
{
}

CPrimitive* CExtendedText::Clone()
{
	CExtendedText* p = new CExtendedText();
	*p = *(this);
	return p;
}

BOOL CExtendedText::operator= (CExtendedText& src)
{
	CPrimitive::operator= ( src);
	m_sText = src.m_sText;
	m_FontFace = src.m_FontFace;
	m_FontHeight = src.m_FontHeight;
	m_FontWidth = src.m_FontWidth;
	m_Flags = src.m_Flags;
	m_Angle = src.m_Angle;

	RecreateFont();
	return TRUE;
}

void CExtendedText::Draw( CDC *pdc, BOOL bXNOR, COLORREF backcolor, COLORREF forecolor, COLORREF selcolor, int zoom, int nLayer, BOOL blackandwhite)
{
	// Se c'e' un punto che vale (-1,-1), qualcuno ha fatto dei poltigli e
	// quindi non disegno nulla
	if( m_ptDefining[0].x == -1 && m_ptDefining[0].y == -1)
		return;

	if( zoom != 100)
		RecreateFont( zoom);

	CFont *oldfont;	
	if( (oldfont = (CFont *)(pdc->SelectObject( &m_Font))) == NULL)
		return;

	// La gestione cambia moltissimo fra spostamento e ridisegno
	if( bXNOR)
	{
		POINT pts[4];
		CSize s = pdc->GetOutputTextExtent( m_sText);
		GetBoundingPoly( pts, s, zoom);

		CPen *oldpen, pen( PS_SOLID, 0, forecolor);
		int oldROP = pdc->m_hAttribDC == NULL ?  R2_COPYPEN : pdc->GetROP2();
		pdc->SetROP2( R2_NOTXORPEN);
		oldpen = pdc->SelectObject( &pen);
		pdc->Polygon( pts, 4);
		pdc->SelectObject( oldpen);
		pdc->SetROP2( oldROP);
		pen.DeleteObject();
	}
	else
	{
		int oldmode = pdc->m_hAttribDC == NULL ? OPAQUE : pdc->GetBkMode();
		COLORREF oldcolor =	pdc->SetTextColor( m_bSelected && !bXNOR ?
			selcolor : forecolor);

		pdc->SetBkMode( TRANSPARENT);

		// Se dobbiamo disegnare il testo swappato dobbiamo creare un memory DC,
		// impostarlo esattamente come il nostro DC, disegnare l testo dritto e infine
		// blittarlo invertito con StretchBlt() nel nostro DC. Boooring!
		if( m_Flags & EXTFONT_STYLE_MIRRORED)
		{
			CFont *oldmfont;
			CDC memdc;
			CBitmap membmp, *oldbmp;

			POINT pts[4];
			CSize s = pdc->GetOutputTextExtent( m_sText);
			s += CSize(2,2);
			GetBoundingPoly( pts, s, zoom);
			CRect destr = GetBoundingPolyRect( pts);
			GetBoundingPoly( pts, s, zoom, TRUE);
			CRect srcr = GetBoundingPolyRect( pts);

			/* Devo creare un bitmap in grado di contenere il testo. Per farlo
				devo calcolare la dimensione un unita` fisiche. */
			CSize psize( srcr.Width()+20, srcr.Height()+20);
			pdc->LPtoDP( &psize);
			memdc.CreateCompatibleDC( pdc);
			membmp.CreateCompatibleBitmap( &memdc, psize.cx, psize.cy);
			oldbmp = (CBitmap*)memdc.SelectObject( &membmp);
			if( pdc->m_hAttribDC != NULL)
			{
				memdc.SetMapMode( pdc->GetMapMode());
				memdc.SetWindowExt( pdc->GetWindowExt());
				memdc.SetViewportExt( pdc->GetViewportExt());
			}
			oldmfont = (CFont*)memdc.SelectObject( &m_Font);
			memdc.SetBkMode( OPAQUE);
			memdc.Rectangle( 0, 0, srcr.Width()+19, srcr.Height()+19);

			/*
				Faccio in modo che nel memory DC il testo venga sempre blittato
				all'inizio. Altrimenti comparirebbero delle bande nere e schifezze simili.
			*/
			CPoint offs( -srcr.left+10, -srcr.top+10);

			memdc.SetTextColor( m_bSelected && !bXNOR ? selcolor : forecolor);
			memdc.TextOut(	m_ptDefining[0].x*zoom/100 + offs.x,
							m_ptDefining[0].y*zoom/100 + offs.y, m_sText);
			
			/* Mi tira il culo trovare la relazione generale, cosi` distingo i casi per
				ognuno dei quattro quadranti */
			int w = srcr.Width();
			int h = srcr.Height();
			if( m_Angle >=0 && m_Angle < 90)
				pdc->StretchBlt(	m_ptDefining[0].x*zoom/100, destr.top, -w, h, &memdc, 
									m_ptDefining[0].x*zoom/100 + offs.x, srcr.top + offs.y, w, h, SRCAND);
			else if( m_Angle >=90 && m_Angle < 180)
				pdc->StretchBlt(	destr.right, destr.top, -w, h, &memdc, 
									srcr.left + offs.x, srcr.top + offs.y, w, h, SRCAND);
			else if( m_Angle >=180 && m_Angle < 270)
				pdc->StretchBlt(	m_ptDefining[0].x*zoom/100, destr.top, w, h, &memdc, 
									m_ptDefining[0].x*zoom/100 + offs.x, srcr.top + offs.y, -w, h, SRCAND);
			else
				pdc->StretchBlt(	destr.right, destr.top, -w, h, &memdc, 
									srcr.left + offs.x, srcr.top + offs.y, w, h, SRCAND);

			memdc.SelectObject( oldmfont);
			memdc.SelectObject( oldbmp);
		}
		else
			pdc->TextOut( m_ptDefining[0].x*zoom/100, m_ptDefining[0].y*zoom/100, m_sText);

		pdc->SetTextColor( oldcolor);
		pdc->SetBkMode( oldmode);
	}

	pdc->SelectObject( oldfont);

	if( zoom != 100)
		RecreateFont();
}

void CExtendedText::UpdateBoundingRgn( CDC *pdc)
{
	CFont *oldfont;	

	if( (oldfont = (CFont *)(pdc->SelectObject( &m_Font))) == NULL)
		return;
	CSize s = pdc->GetOutputTextExtent( m_sText);
	pdc->SelectObject( oldfont);

	/*
		GetOutputTextExtent fa schifo, e non considera le rotazioni. Ma
		dato che noi siamo bravi e conosciamo la trigonometria, non
		e` un grosso problema.
	*/
	POINT pts[4];
	GetBoundingPoly( pts, s);

	// Crea la nuova regione
	m_Region.DeleteObject();
	m_Region.CreatePolygonRgn( pts, 4, ALTERNATE);
}

void CExtendedText::Rotate( CPoint pt, BOOL bCounterClockWise)
{
	CPrimitive::Rotate( pt, bCounterClockWise);

	if( bCounterClockWise && (m_Flags & EXTFONT_STYLE_MIRRORED) == 0 ||
		!bCounterClockWise && (m_Flags & EXTFONT_STYLE_MIRRORED) != 0)
		m_Angle += 90;
	else
		m_Angle -= 90;

	while( m_Angle < 0)
		m_Angle += 360;
	while( m_Angle > 360)
		m_Angle -= 360;

	RecreateFont();
}

void CExtendedText::Swap( int xPos)
{
	CPrimitive::Swap( xPos);

	if( m_Flags & EXTFONT_STYLE_MIRRORED)
		m_Flags &= ~EXTFONT_STYLE_MIRRORED;
	else
		m_Flags |= EXTFONT_STYLE_MIRRORED;
}

int CExtendedText::GetNumDefPoints( void)
{
	return 1;
}

DWORD CExtendedText::GetProp( void)
{
	return PROP_ROTATEFULL | PROP_SWAPFULL | PROP_UPDATEONZOOM | PROP_NEEDPROPS | PROP_ERASEONSELECT;
}

BOOL CExtendedText::GetAttribName( int index, CString& name, CString& values)
{
	switch( index)
	{
		case 0:
			name = "Text";
			values = "#STRING";
			return TRUE;

		case 1:
			name = "FontFace";
			values = TrovaFontFissi();
			return TRUE;

		case 2:
			name = "FontHeight";
			values = "#INT 2 100";
			return TRUE;

		case 3:
			name = "FontWidth";
			values = "#INT 2 100";
			return TRUE;

		case 4:
			name = "RotationAngle";
			values = "#INT 0 359";
			return TRUE;

		case 5:
			name = "bBold";
			values = "(Si No)";
			return TRUE;

		case 6:
			name = "bItalic";
			values = "(Si No)";
			return TRUE;

		case 7:
			name = "bMirrored";
			values = "(Si No)";
			return TRUE;

		case 8:
			name = "bUnderlined";
			values = "(Si No)";
			return TRUE;

		default:
			return FALSE;
	}
}

CString CExtendedText::TrovaFontFissi()
{
	CClientDC dc( AfxGetMainWnd());
	CString lista = "(";
	LOGFONT lf;

	lf.lfCharSet = ANSI_CHARSET;
	lstrcpy( lf.lfFaceName, "");
	lf.lfPitchAndFamily = 0;
	EnumFontFamiliesEx( dc.m_hDC, &lf, (FONTENUMPROC)EnumFontFamExProc, (LPARAM)&lista, 0);
	lista += ")";

	return lista;
}

int CALLBACK CExtendedText::EnumFontFamExProc( ENUMLOGFONTEX *lpelfe,
	NEWTEXTMETRICEX *lpntme, int FontType, LPARAM lParam)
{
	CString* lista = (CString*)lParam;
	
	if( FontType != TRUETYPE_FONTTYPE ||
		(lpelfe->elfLogFont.lfPitchAndFamily & FIXED_PITCH) != FIXED_PITCH)
			return 1;
	
	CString pippo = lpelfe->elfFullName;
	CPrimitive::Rimpiazza( pippo, " ", "++");
	if( *lista != "(")
		*lista += " ";
	*lista += pippo;
	return 1;
}

BOOL CExtendedText::GetAttrib( const char *name, CString& value)
{
	if( !lstrcmp( name, "Text"))
		value = m_sText;
	else if( !lstrcmp( name, "FontFace"))
		value = m_FontFace;
	else if( !lstrcmp( name, "FontHeight"))
		value.Format( "%d", m_FontHeight);
	else if( !lstrcmp( name, "FontWidth"))
		value.Format( "%d", m_FontWidth);
	else if( !lstrcmp( name, "RotationAngle"))
		value.Format( "%d", m_Angle);
	else if( !lstrcmp( name, "bBold"))
		value = (m_Flags & EXTFONT_STYLE_BOLD) ? "Si" : "No";
	else if( !lstrcmp( name, "bItalic"))
		value = (m_Flags & EXTFONT_STYLE_ITALIC) ? "Si" : "No";
	else if( !lstrcmp( name, "bMirrored"))
		value = (m_Flags & EXTFONT_STYLE_MIRRORED) ? "Si" : "No";
	else if( !lstrcmp( name, "bUnderlined"))
		value = (m_Flags & EXTFONT_STYLE_UNDERLINED) ? "Si" : "No";
	else
		return FALSE;

	return TRUE;
}

BOOL CExtendedText::SetAttrib( const char *name, const char *value)
{
	int n = atoi( value);

	if( !lstrcmp( name, "Text"))
		m_sText = value;
	else if( !lstrcmp( name, "FontFace"))
		m_FontFace = value;
	else if( !lstrcmp( name, "FontHeight"))
	{
		if( n < 2 || n > 100)
			return FALSE;
		m_FontHeight = n;
	}
	else if( !lstrcmp( name, "FontWidth"))
	{
		if( n < 2 || n > 100)
			return FALSE;
		m_FontWidth = n;
	}
	else if( !lstrcmp( name, "RotationAngle"))
	{
		if( n < 0 || n > 359)
			return FALSE;
		m_Angle = n;
	}
	else if( !lstrcmp( name, "bBold"))
	{
		if( !lstrcmp( value, "Si"))
			m_Flags |= EXTFONT_STYLE_BOLD;
		else if( !lstrcmp( value, "No"))
			m_Flags &= ~EXTFONT_STYLE_BOLD;
		else
			return FALSE;
	}
	else if( !lstrcmp( name, "bItalic"))
	{
		if( !lstrcmp( value, "Si"))
			m_Flags |= EXTFONT_STYLE_ITALIC;
		else if( !lstrcmp( value, "No"))
			m_Flags &= ~EXTFONT_STYLE_ITALIC;
		else
			return FALSE;
	}
	else if( !lstrcmp( name, "bMirrored"))
	{
		if( !lstrcmp( value, "Si"))
			m_Flags |= EXTFONT_STYLE_MIRRORED;
		else if( !lstrcmp( value, "No"))
			m_Flags &= ~EXTFONT_STYLE_MIRRORED;
		else
			return FALSE;
	}
	else if( !lstrcmp( name, "bUnderlined"))
	{
		if( !lstrcmp( value, "Si"))
			m_Flags |= EXTFONT_STYLE_UNDERLINED;
		else if( !lstrcmp( value, "No"))
			m_Flags &= ~EXTFONT_STYLE_UNDERLINED;
		else
			return FALSE;
	}
	else
		return FALSE;

	/* Backup nuove impostazioni */
	_lastText = *this;

	RecreateFont();
	return TRUE;
}

CPrimitive* CExtendedText::Load( CString& linea)
{
	linea.TrimLeft();

	CString cmp = linea.Left(2);
	cmp.MakeUpper();
	if( cmp != "TX" && cmp != "TY" && cmp != "TE")
		return NULL;

	char str[256];
	CExtendedText* prim = new CExtendedText();
	prim->m_nLayer = 0;
	CString buf = linea.Right( linea.GetLength()-2);
	
	if( cmp == "TX")
		sscanf( LPCSTR(buf), "%d %d %d %d %d %u %s", &(prim->m_ptDefining[0].x),
			&(prim->m_ptDefining[0].y), &(prim->m_FontHeight), &(prim->m_FontWidth),
			&(prim->m_Angle), &(prim->m_Flags), str);
	else if( cmp == "TY")
		sscanf( LPCSTR(buf), "%d %d %d %d %d %u %d %s", &(prim->m_ptDefining[0].x),
			&(prim->m_ptDefining[0].y), &(prim->m_FontHeight), &(prim->m_FontWidth),
			&(prim->m_Angle), &(prim->m_Flags), &(prim->m_nLayer), str);
	else
	{
		sscanf( LPCSTR(buf), "%d %d", &(prim->m_ptDefining[0].x),
			&(prim->m_ptDefining[0].y));
		prim->m_FontHeight = 5;
		prim->m_FontWidth = 3;
		prim->m_Angle = 0;
		prim->m_Flags = 0;
		lstrcpy( str, "*");
	}

	prim->m_FontFace = (str=="*" ? "Courier New" : str);
	CPrimitive::Rimpiazza( prim->m_FontFace, "++", " ");
	prim->m_sText = ExtractString( buf, cmp=="TX" ? 7 : cmp=="TY" ? 8 : 2);

	if( !prim->IsProper())
	{
		delete prim;
		return NULL;
	}

	prim->SaveState();
	prim->RecreateFont();
	return prim;
}

BOOL CExtendedText::Save( CString& str, BOOL bSplit)
{
	CString ff = m_FontFace;
	CPrimitive::Rimpiazza( ff, " ", "++");
	if( ff == "Courier++New")
		ff = "*";

	str.Format( "TY %d %d %d %d %d %u %d %s %s\r\n", m_ptDefining[0].x, m_ptDefining[0].y,
		m_FontHeight, m_FontWidth, m_Angle, m_Flags, m_nLayer, ff, m_sText);

	return TRUE;
}

void CExtendedText::Serialize( CArchive &ar)
{
	// TODO

	if( ar.IsStoring())
	{
		ar << m_ptDefining[0] << m_sText;
	}
	else
	{
		ar >> m_ptDefining[0] >> m_sText;
	}
}


///////////////////////////////////////////////////////////////////////////////////
// CConnection
///////////////////////////////////////////////////////////////////////////////////

CConnection::CConnection()
{
}

CConnection::CConnection( CPoint start, BOOL bConst)
{
	m_ptDefining[0] = m_ptSaved[0] = start;
	m_bConst = bConst;
}

CConnection::~CConnection()
{
}

CPrimitive* CConnection::Clone()
{
	CConnection* p = new CConnection();
	*p = *(this);
	return p;
}

void CConnection::Draw( CDC *pdc, BOOL bXNOR, COLORREF backcolor, COLORREF forecolor, COLORREF selcolor, int zoom, int nLayer, BOOL blackandwhite)
{
	// Se c'e' un punto che vale (-1,-1), qualcuno ha fatto dei poltigli e
	// quindi non disegno nulla
	if( m_ptDefining[0].x == -1 && m_ptDefining[0].y == -1)
		return;

	CPen *oldpen, pen( PS_SOLID, 0, m_bSelected && !bXNOR ? selcolor : forecolor);
	oldpen = pdc->SelectObject( &pen);

	if( bXNOR)
	{
		int oldROP = pdc->m_hAttribDC == NULL ?  R2_COPYPEN : pdc->GetROP2();
		pdc->SetROP2( R2_NOTXORPEN);

		pdc->Arc( m_ptDefining[0].x-1, m_ptDefining[0].y-1,
			m_ptDefining[0].x+1, m_ptDefining[0].y+1, m_ptDefining[0].x, 
			m_ptDefining[0].y, m_ptDefining[0].x, m_ptDefining[0].y);

		pdc->SetROP2( oldROP);
	}
	else
	{
		CBrush brush( m_bSelected && !bXNOR ? selcolor : forecolor);
		CBrush *oldbrush;
		oldbrush = pdc->SelectObject( &brush);

		pdc->Ellipse( (m_ptDefining[0].x-1)*zoom/100, (m_ptDefining[0].y-1)*zoom/100,
			(m_ptDefining[0].x+1)*zoom/100, (m_ptDefining[0].y+1)*zoom/100);

		pdc->SelectObject( oldbrush);
		brush.DeleteObject();
	}

	pdc->SelectObject( oldpen);
	pen.DeleteObject();	
}

void CConnection::UpdateBoundingRgn( CDC *pdc)
{
	m_Region.DeleteObject();
	m_Region.CreateRectRgn( m_ptDefining[0].x-1, m_ptDefining[0].y-1,
			m_ptDefining[0].x+1, m_ptDefining[0].y+1);
}

int CConnection::GetNumDefPoints( void)
{
	return 1;
}

DWORD CConnection::GetProp( void)
{
	return PROP_ROTATEFULL | PROP_SWAPFULL;
}

CPrimitive* CConnection::Load( CString& linea)
{
	linea.TrimLeft();
	CString cmp = linea.Left(2);
	cmp.MakeUpper();

	if( cmp != "SA")
		return NULL;
	
	CConnection* prim = new CConnection();
	prim->m_nLayer = 0;
	CString buf = linea.Right( linea.GetLength()-2);
	sscanf( LPCSTR(buf), "%d %d %d", &(prim->m_ptDefining[0].x),
		&(prim->m_ptDefining[0].y), &(prim->m_nLayer));

	if( !prim->IsProper())
	{
		delete prim;
		return NULL;
	}
	
	prim->SaveState();
	return prim;
}

BOOL CConnection::Save( CString& str, BOOL bSplit)
{
	str.Format( "SA %d %d", m_ptDefining[0].x, m_ptDefining[0].y);

	if( m_nLayer == 0)
		str += "\r\n";
	else
	{
		CString l;
		l.Format( " %d\r\n", m_nLayer);
		str += l;
	}

	return TRUE;
}

void CConnection::Serialize( CArchive &ar)
{
	if( ar.IsStoring())
	{
		ar << m_ptDefining[0];
	}
	else
	{
		ar >> m_ptDefining[0];
	}
}



///////////////////////////////////////////////////////////////////////////////////
// CBezier
///////////////////////////////////////////////////////////////////////////////////

CBezier::CBezier()
{
}

CBezier::CBezier( CPoint p1, CPoint p2, CPoint p3, CPoint p4, BOOL bConst)
{
	m_ptDefining[0] = m_ptSaved[0] = p1;
	m_ptDefining[1] = m_ptSaved[1] = p2;
	m_ptDefining[2] = m_ptSaved[2] = p3;
	m_ptDefining[3] = m_ptSaved[3] = p4;
	m_bConst = bConst;
}

CBezier::~CBezier()
{
}

CPrimitive* CBezier::Clone()
{
	CBezier* p = new CBezier();
	*p = *(this);
	return p;
}

void CBezier::Draw( CDC *pdc, BOOL bXNOR, COLORREF backcolor, COLORREF forecolor, COLORREF selcolor, int zoom, int nLayer, BOOL blackandwhite)
{
	// Se nessun punto e' definito, non disegno nulla; se non tutti i punti
	// sono definiti, disegno una spezzata; se tutti i punti sono definiti,
	// disegno la Bezier curve (ooooh! :-)).
	if( m_ptDefining[0].x == -1 && m_ptDefining[0].y == -1 ||
		m_ptDefining[1].x == -1 && m_ptDefining[1].y == -1)
		return;

	CPen *oldpen, pen( PS_SOLID, 0, m_bSelected && !bXNOR ? selcolor : forecolor);
	int oldROP = pdc->m_hAttribDC == NULL ?  R2_COPYPEN : pdc->GetROP2();

	if( bXNOR)
		pdc->SetROP2( R2_NOTXORPEN);
	else
		pdc->SetROP2( R2_COPYPEN);

	oldpen = pdc->SelectObject( &pen);

	if( m_ptDefining[3].x != -1 || m_ptDefining[3].y != -1)
	{
		// Bezier!
		CPoint ptDefining[4];
		for( int a=0; a<4; a++)
			ptDefining[a] = CPoint(m_ptDefining[a].x*zoom/100,m_ptDefining[a].y*zoom/100);

		pdc->PolyBezier( ptDefining, 4);
	}
	else
	{
		// Spezzata!
		pdc->MoveTo( m_ptDefining[0]);
		for( int a=1; a<3; a++)
		{
			if( m_ptDefining[a].x != -1 || m_ptDefining[a].y != -1)
				pdc->LineTo( m_ptDefining[a]);
		}
	}

	pdc->SelectObject( oldpen);
	pen.DeleteObject();
	pdc->SetROP2( oldROP);
}

int CBezier::GetNumDefPoints( void)
{
	return 4;
}

DWORD CBezier::GetProp( void)
{
	return PROP_ROTATEFULL | PROP_SWAPFULL | PROP_UPDATEONZOOM;
}

void CBezier::UpdateBoundingRgn( CDC *pdc)
{
	// Implementazione manuale (imprecisa ma meglio di niente)
	double b03, b13, b23, b33;
	CPoint ptold = m_ptDefining[0];
	CPoint p;
	CRgn rgn;
	// rgn.CreateRectRgn( 0, 0, 1, 1);

	// Secca la vecchia regione
	HRGN rg;
	rg = (HRGN)(m_Region.Detach());
	if( rg != NULL)
		::DeleteObject( rg);

	// Le curve sono composte da 5 segmenti (1.01 anziche' 1.0 e' per prevenire
	// errori di arrotondamento)
	for( double u = 0; u < 1.01; u += 0.2)
	{		
		b03 = pow(1 - u, 3);
		b13 = 3 * u * pow(1 - u, 2);
		b23 = 3 * pow(u, 2) * (1 - u);
		b33 = pow(u, 3);

		p.x = (int)(	m_ptDefining[0].x * b03 +
						m_ptDefining[1].x * b13 +
						m_ptDefining[2].x * b23 +
						m_ptDefining[3].x * b33);
		p.y = (int)(	m_ptDefining[0].y * b03 +
						m_ptDefining[1].y * b13 +
						m_ptDefining[2].y * b23 +
						m_ptDefining[3].y * b33);

		// Unione della regione (crea la prima volta)
		CPrimitive::RegioneLinea( pdc, rgn, ptold, p, SELECTOR_SIZE);
		if( m_Region.m_hObject == NULL)
		{
			m_Region.CreateRectRgn( 0, 0, 1, 1);
			m_Region.CopyRgn( &rgn);
		}
		else
			m_Region.CombineRgn( &m_Region, &rgn, RGN_OR);

		ptold = p;
	}
}

CPrimitive* CBezier::Load( CString& linea)
{
	linea.TrimLeft();
	CString cmp = linea.Left(2);
	cmp.MakeUpper();

	if( cmp != "BE")
		return NULL;
	
	CBezier* prim = new CBezier();
	prim->m_nLayer = 0;
	CString buf = linea.Right( linea.GetLength()-2);
	sscanf( LPCSTR(buf), "%d %d %d %d %d %d %d %d %d",
		&(prim->m_ptDefining[0].x), &(prim->m_ptDefining[0].y),
		&(prim->m_ptDefining[1].x), &(prim->m_ptDefining[1].y),
		&(prim->m_ptDefining[2].x), &(prim->m_ptDefining[2].y),
		&(prim->m_ptDefining[3].x), &(prim->m_ptDefining[3].y), &(prim->m_nLayer));

	if( !prim->IsProper())
	{
		delete prim;
		return NULL;
	}
	
	prim->SaveState();
	return prim;
}

BOOL CBezier::Save( CString& str, BOOL bSplit)
{
	str.Format( "BE %d %d %d %d %d %d %d %d",
		m_ptDefining[0].x, m_ptDefining[0].y,
		m_ptDefining[1].x, m_ptDefining[1].y,
		m_ptDefining[2].x, m_ptDefining[2].y,
		m_ptDefining[3].x, m_ptDefining[3].y);

	if( m_nLayer == 0)
		str += "\r\n";
	else
	{
		CString l;
		l.Format( " %d\r\n", m_nLayer);
		str += l;
	}

	return TRUE;
}

void CBezier::Serialize( CArchive &ar)
{
	if( ar.IsStoring())
	{
		ar << m_ptDefining[0] << m_ptDefining[1] <<
		m_ptDefining[2] << m_ptDefining[3];
	}
	else
	{
		ar >> m_ptDefining[0] >> m_ptDefining[1] >>
		m_ptDefining[2] >> m_ptDefining[3];
	}
}



///////////////////////////////////////////////////////////////////////////////////
// CPoly
///////////////////////////////////////////////////////////////////////////////////

CPoly::CPoly()
{
}

CPoly::CPoly( CPoint p1, CPoint p2, CPoint p3, BOOL bConst)
{
	// Pesca ultimi settaggi dal pool
	m_bFilled = _lastPoly.m_bFilled;

	m_ptDefining[0] = m_ptSaved[0] = p1;
	m_ptDefining[1] = m_ptSaved[1] = p2;
	m_ptDefining[2] = m_ptSaved[2] = p3;
	m_bConst = bConst;
	m_nVirtualPts = MAX_DEFINING_POINTS;
}

CPoly::~CPoly()
{
}

CPrimitive* CPoly::Clone()
{
	CPoly* p = new CPoly();
	*p = *(this);
	return p;
}

BOOL CPoly::operator= (CPoly& src)
{
	CPrimitive::operator=(src);
	m_nVirtualPts = src.m_nVirtualPts;
	m_bFilled = src.m_bFilled;
	return TRUE;
}

void CPoly::Draw( CDC *pdc, BOOL bXNOR, COLORREF backcolor, COLORREF forecolor, COLORREF selcolor, int zoom, int nLayer, BOOL blackandwhite)
{
	// Se sono definiti meno di sue punti, non disegno nulla; se sono definiti due
	// punti, disegno una linea; se sono definiti piu' di due punti e 
	// m_nVirtualPts==MAX_DEFINING_POINTS, disegno il poligono
	// con n-1 lati e quello con n lati (per rappresentare l'alternativa);
	// se infine n>2 e m_nVirtualPts e' gia' stato ridotto, disegno soltanto il
	// poligono "giusto".
	if( m_ptDefining[1].x == -1 && m_ptDefining[1].y == -1)
		return;

	CPen *oldpen, pen( PS_SOLID, 0, m_bSelected && !bXNOR ? selcolor : forecolor);
	int oldROP = pdc->m_hAttribDC == NULL ?  R2_COPYPEN : pdc->GetROP2();
	oldpen = pdc->SelectObject( &pen);

	if( bXNOR)
	{
		pdc->SetROP2( R2_NOTXORPEN);
		pdc->MoveTo( m_ptDefining[0]);
		pdc->LineTo( m_ptDefining[1]);

		int a;
		for( a=2; a<m_nVirtualPts; a++)
		{
			if( m_ptDefining[a].x != -1 || m_ptDefining[a].y != -1)
				pdc->LineTo( m_ptDefining[a]);
			else
				break;
		}
		
		if( a>2)
			pdc->LineTo( m_ptDefining[0]);
		
		if( m_nVirtualPts==MAX_DEFINING_POINTS && a>3 &&
			 m_ptDefining[a-2] != m_ptDefining[a-1] &&
			 a<m_nVirtualPts)
		{
			pdc->MoveTo( m_ptDefining[a-2]);
			pdc->LineTo( m_ptDefining[0]);
		}
	}
	else
	{
		CBrush *oldbrush, brush;
		
		if( !m_bFilled || bXNOR)
			brush.Attach( ::GetStockObject( NULL_BRUSH));
		else
			brush.CreateSolidBrush( m_bSelected ? selcolor : forecolor);
		oldbrush = pdc->SelectObject( &brush);

		pdc->SetROP2( R2_COPYPEN);

		CPoint ptDefining[MAX_DEFINING_POINTS+1];
		for( int a=0; a<m_nVirtualPts; a++)
			ptDefining[a] = CPoint(m_ptDefining[a].x*zoom/100,m_ptDefining[a].y*zoom/100);
		pdc->Polygon( ptDefining, m_nVirtualPts);

		pdc->SelectObject( oldbrush);
		brush.DeleteObject();
	}

	pdc->SelectObject( oldpen);
	pen.DeleteObject();
	pdc->SetROP2( oldROP);
}

int CPoly::GetNumDefPoints( void)
{
	return m_nVirtualPts;
}

void CPoly::SetPoint( int nIndex, CPoint pt, BOOL bSignal)
{
	if( bSignal && nIndex > 2 && m_ptDefining[nIndex-1] == pt)
		m_nVirtualPts = nIndex;
	else
		CPrimitive::SetPoint( nIndex, pt);
}

void CPoly::UpdateBoundingRgn( CDC *pdc)
{
	HRGN rg;
	rg = (HRGN)(m_Region.Detach());
	if( rg != NULL)
		::DeleteObject( rg);

	// Calcolo l'ingombro del poligono
	int x, y;
	int minx=DEFAULT_AREAX, miny=DEFAULT_AREAY;
	int maxx=-DEFAULT_AREAX, maxy=-DEFAULT_AREAY;
	for( int a=0; a<m_nVirtualPts; a++)
	{
		x = m_ptDefining[a].x;
		y = m_ptDefining[a].y;
		if( x < minx)
			minx = x;
		if( x > maxx)
			maxx = x;
		if( y < miny)
			miny = y;
		if( y > maxy)
			maxy = y;
	}

	// Se il poligono e' piccolo, lo avvolgo con un rettangolo; altrimenti se e'
	// vuoto uso l'outline, se e' pieno CreatePolygonRgn
	if( maxx-minx < 10 || maxy-miny < 10)
		m_Region.CreateRectRgn( minx, miny, maxx, maxy);
	else if( !m_bFilled)
	{
		int a;
		CRgn rgn;
		CPoint pt, ptold = m_ptDefining[m_nVirtualPts-1];

		// rgn.CreateRectRgn( 0, 0, 1, 1);
		for( a=0; a<m_nVirtualPts; a++)
		{
			pt = m_ptDefining[a];
			CPrimitive::RegioneLinea( pdc, rgn, ptold, pt, SELECTOR_SIZE);
			ptold = pt;
			if( m_Region.m_hObject == NULL)
			{
				m_Region.CreateRectRgn( 0, 0, 1, 1);
				m_Region.CopyRgn( &rgn);
			}
			else
				m_Region.CombineRgn( &m_Region, &rgn, RGN_OR);
		}
	}
	else
		m_Region.CreatePolygonRgn( m_ptDefining, m_nVirtualPts, ALTERNATE);
}

DWORD CPoly::GetProp( void)
{
	return PROP_ROTATEFULL | PROP_SWAPFULL;
}

BOOL CPoly::GetAttribName( int index, CString& name, CString& values)
{
	if( index != 0)
		return FALSE;

	name = "Filled";
	values = "(Si No)";
	return TRUE;
}

BOOL CPoly::GetAttrib( const char *name, CString& value)
{
	if( lstrcmp(name, "Filled"))
		return FALSE;

	value = m_bFilled ? "Si" : "No";
	return TRUE;
}

BOOL CPoly::SetAttrib( const char *name, const char *value)
{
	if( lstrcmp( name, "Filled"))
		return FALSE;

	if( !lstrcmp( value, "Si"))
		m_bFilled = TRUE;
	else if( !lstrcmp( value, "No"))
		m_bFilled = FALSE;
	else
		return FALSE;

	/* Backup nuove impostazioni */
	_lastPoly = *this;

	return TRUE;
}

CPrimitive* CPoly::Load( CString& linea)
{
	linea.TrimLeft();
	linea.TrimRight();
	CString cmp = linea.Left(2);
	cmp.MakeUpper();

	if( cmp != "PP" && cmp != "PV")
		return NULL;
	
	CPoly* prim = new CPoly();
	prim->m_nLayer = 0;
	prim->m_bFilled = (cmp=="PP") ? TRUE : FALSE;
	CString buf = linea.Right( linea.GetLength()-2);

	int nPts = sscanf( LPCSTR(buf), "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
		&(prim->m_ptDefining[0].x), &(prim->m_ptDefining[0].y),
		&(prim->m_ptDefining[1].x), &(prim->m_ptDefining[1].y),
		&(prim->m_ptDefining[2].x), &(prim->m_ptDefining[2].y),
		&(prim->m_ptDefining[3].x), &(prim->m_ptDefining[3].y),
		&(prim->m_ptDefining[4].x), &(prim->m_ptDefining[4].y),
		&(prim->m_ptDefining[5].x), &(prim->m_ptDefining[5].y),
		&(prim->m_ptDefining[6].x), &(prim->m_ptDefining[6].y),
		&(prim->m_ptDefining[7].x), &(prim->m_ptDefining[7].y),
		&(prim->m_ptDefining[8].x), &(prim->m_ptDefining[8].y),		
		&(prim->m_ptDefining[9].x), &(prim->m_ptDefining[9].y), &(prim->m_nLayer));

	prim->m_nVirtualPts = nPts / 2;

	/*
		Se abbiamo un numero di coordinate dispari, l'ultima e` il layer. C'e`
		il caso particolare in cui siano stati specificati 21 punti: in questo
		caso il layer e` gia` stato impostato correttamente.
	 */
	if( nPts % 2 != 0 && nPts < 21)
	{
		prim->m_nLayer = prim->m_ptDefining[nPts/2].x;
		prim->m_ptDefining[nPts/2].x = -1;
	}

	if( !prim->IsProper())
	{
		delete prim;
		return NULL;
	}
	
	prim->SaveState();
	return prim;
}

BOOL CPoly::Save( CString& str, BOOL bSplit)
{
	str.Format( "%s %d %d %d %d %d %d",
		m_bFilled ? "PP" : "PV",
		m_ptDefining[0].x, m_ptDefining[0].y,
		m_ptDefining[1].x, m_ptDefining[1].y,
		m_ptDefining[2].x, m_ptDefining[2].y);

	int a=3;
	CString pt;
	while( m_nVirtualPts > a)
	{
		pt.Format( " %d %d", m_ptDefining[a].x, m_ptDefining[a].y);
		str += pt;
		a++;
	}

	/*
		Possiamo riconoscere la specificazione del layer dal fatto che aggiungendolo
		abbiamo un numero dispari di coordinate.
	*/
	if( m_nLayer == 0)
		str += "\r\n";
	else
	{
		CString l;
		l.Format( " %d\r\n", m_nLayer);
		str += l;
	}

	return TRUE;
}

void CPoly::Serialize( CArchive &ar)
{
	if( ar.IsStoring())
	{
		// TODO (future)
	}
	else
	{
		// TODO (future)
	}
}


///////////////////////////////////////////////////////////////////////////////////
// CEllipse
///////////////////////////////////////////////////////////////////////////////////

CEllipse::CEllipse()
{
}

CEllipse::CEllipse( CPoint start, CPoint stop, BOOL bConst)
{
	// Pesca ultimi settaggi dal pool
	*this = _lastEllipse;

	m_ptDefining[0] = m_ptSaved[0] = start;
	m_ptDefining[1] = m_ptSaved[1] = stop;
	m_bConst = bConst;
}

CEllipse::~CEllipse()
{
}

CPrimitive* CEllipse::Clone()
{
	CEllipse* p = new CEllipse();
	*p = *(this);
	return p;
}

BOOL CEllipse::operator= (CEllipse& src)
{
	CPrimitive::operator=(src);
	m_bFilled = src.m_bFilled;
	return TRUE;
}

void CEllipse::Draw( CDC *pdc, BOOL bXNOR, COLORREF backcolor, COLORREF forecolor, COLORREF selcolor, int zoom, int nLayer, BOOL blackandwhite)
{
	// Se c'e' un punto che vale (-1,-1), qualcuno ha fatto dei poltigli e
	// quindi non disegno nulla
	if( m_ptDefining[0].x == -1 && m_ptDefining[0].y == -1 ||
		m_ptDefining[1].x == -1 && m_ptDefining[1].y == -1)
		return;

	CPen *oldpen, pen( PS_SOLID, 0, m_bSelected && !bXNOR ? selcolor : forecolor);
	int oldROP = pdc->m_hAttribDC == NULL ?  R2_COPYPEN : pdc->GetROP2();
	CBrush *oldbrush, brush;

	if( bXNOR)
		pdc->SetROP2( R2_NOTXORPEN);
	else
		pdc->SetROP2( R2_COPYPEN);

	oldpen = pdc->SelectObject( &pen);
	if( !m_bFilled || bXNOR)
		brush.Attach( ::GetStockObject( NULL_BRUSH));
	else
		brush.CreateSolidBrush( m_bSelected ? selcolor : forecolor);
	oldbrush = pdc->SelectObject( &brush);

	pdc->Ellipse( m_ptDefining[0].x*zoom/100, m_ptDefining[0].y*zoom/100,
		m_ptDefining[1].x*zoom/100, m_ptDefining[1].y*zoom/100);

	pdc->SelectObject( oldbrush);
	pdc->SelectObject( oldpen);
	pen.DeleteObject();
	brush.DeleteObject();
	pdc->SetROP2( oldROP);
}

int CEllipse::GetNumDefPoints( void)
{
	return 2;
}

DWORD CEllipse::GetProp( void)
{
	return PROP_ROTATEFULL | PROP_SWAPFULL | PROP_UPDATEONZOOM;
}

BOOL CEllipse::GetAttribName( int index, CString& name, CString& values)
{
	if( index != 0)
		return FALSE;

	name = "Filled";
	values = "(Si No)";
	return TRUE;
}

BOOL CEllipse::GetAttrib( const char *name, CString& value)
{
	if( lstrcmp(name, "Filled"))
		return FALSE;

	value = m_bFilled ? "Si" : "No";
	return TRUE;
}

BOOL CEllipse::SetAttrib( const char *name, const char *value)
{
	if( lstrcmp( name, "Filled"))
		return FALSE;

	if( !lstrcmp( value, "Si"))
		m_bFilled = TRUE;
	else if( !lstrcmp( value, "No"))
		m_bFilled = FALSE;
	else
		return FALSE;

	/* Backup nuove impostazioni */
	_lastEllipse = *this;

	return TRUE;
}

void CEllipse::UpdateBoundingRgn( CDC *pdc)
{
	HRGN rg;
	rg = (HRGN)(m_Region.Detach());
	if( rg != NULL)
		::DeleteObject( rg);

	// Windows '45 non riconosce le ellissi nei path, cosi' devo definire a
	// mano le regioni
	if( ABS(m_ptDefining[0].x-m_ptDefining[1].x) < 10 ||
		ABS(m_ptDefining[0].y-m_ptDefining[1].y) < 10)
			m_Region.CreateRectRgn( m_ptDefining[0].x, m_ptDefining[0].y,
					m_ptDefining[1].x, m_ptDefining[1].y);
	else if( !m_bFilled)
	{
		// TODO
		int x, y;
		int centrox = (m_ptDefining[0].x+m_ptDefining[1].x)/2;
		int centroy = (m_ptDefining[0].y+m_ptDefining[1].y)/2;
		int a = ABS(m_ptDefining[1].x-m_ptDefining[0].x)/2;
		int b = ABS(m_ptDefining[1].y-m_ptDefining[0].y)/2;
		double a2 = (double)a*a;
		double b2 = (double)b*b;
		double ab2 = a2/b2;
		CPoint pt1, pt2, ptold1, ptold2;
		CRgn rgn;
		int step = b*2/7;

		if( step < 1)
			step = 1;

		for( y=-b; y<=b; y+=step)
		{	
			x = (int)sqrt( ab2 * (b2-(double)y*y));
			pt1.x = centrox-x;
			pt1.y = centroy+y;
			if( y != -b)
			{
				CPrimitive::RegioneLinea( pdc, rgn, ptold1, pt1, SELECTOR_SIZE);
				if( m_Region.m_hObject == NULL)
				{
					m_Region.CreateRectRgn( 0, 0, 1, 1);
					m_Region.CopyRgn( &rgn);
				}
				else
					m_Region.CombineRgn( &m_Region, &rgn, RGN_OR);
			}
			ptold1 = pt1;

			pt2.x = centrox+x;
			pt2.y = centroy+y;
			if( y != -b)
			{
				CPrimitive::RegioneLinea( pdc, rgn, ptold2, pt2, SELECTOR_SIZE);
				if( m_Region.m_hObject == NULL)
				{
					m_Region.CreateRectRgn( 0, 0, 1, 1);
					m_Region.CopyRgn( &rgn);
				}
				else
					m_Region.CombineRgn( &m_Region, &rgn, RGN_OR);
			}
			ptold2 = pt2;

			if( y != b && y+step > b)
				y = b-step;
		}
	}
	else
		m_Region.CreateEllipticRgn( m_ptDefining[0].x, m_ptDefining[0].y,
			m_ptDefining[1].x, m_ptDefining[1].y);
}

CPrimitive* CEllipse::Load( CString& linea)
{
	linea.TrimLeft();
	CString cmp = linea.Left(2);
	cmp.MakeUpper();

	if( cmp != "EP" && cmp != "EV")
		return NULL;
	
	CEllipse* prim = new CEllipse();
	prim->m_nLayer = 0;
	prim->m_bFilled = (cmp=="EP") ? TRUE : FALSE;

	CString buf = linea.Right( linea.GetLength()-2);
	sscanf( LPCSTR(buf), "%d %d %d %d %d", &(prim->m_ptDefining[0].x),
		&(prim->m_ptDefining[0].y), &(prim->m_ptDefining[1].x),
		&(prim->m_ptDefining[1].y), &(prim->m_nLayer) );

	if( !prim->IsProper())
	{
		delete prim;
		return NULL;
	}
	
	prim->SaveState();
	return prim;
}

BOOL CEllipse::Save( CString& str, BOOL bSplit)
{
	str.Format( "%s %d %d %d %d",
		m_bFilled ? "EP" : "EV",
		m_ptDefining[0].x, m_ptDefining[0].y,
		m_ptDefining[1].x, m_ptDefining[1].y);

	if( m_nLayer == 0)
		str += "\r\n";
	else
	{
		CString l;
		l.Format( " %d\r\n", m_nLayer);
		str += l;
	}

	return TRUE;
}

void CEllipse::Serialize( CArchive &ar)
{
	if( ar.IsStoring())
	{
		ar <<	m_ptDefining[0] << m_ptDefining[1];
	}
	else
	{
		ar >>	m_ptDefining[0] >> m_ptDefining[1];
	}
}


///////////////////////////////////////////////////////////////////////////////////
// CRectangle
///////////////////////////////////////////////////////////////////////////////////

CRectangle::CRectangle()
{
}

CRectangle::CRectangle( CPoint start, CPoint stop, BOOL bConst)
{
	// Pesca ultimi settaggi dal pool
	*this = _lastRect;

	m_ptDefining[0] = m_ptSaved[0] = start;
	m_ptDefining[1] = m_ptSaved[1] = stop;
	m_bConst = bConst;
}

CRectangle::~CRectangle()
{
}

CPrimitive* CRectangle::Clone()
{
	CRectangle* p = new CRectangle();
	*p = *(this);
	return p;
}

BOOL CRectangle::operator= (CRectangle& src)
{
	CPrimitive::operator=(src);
	m_bFilled = src.m_bFilled;
	return TRUE;
}

void CRectangle::Draw( CDC *pdc, BOOL bXNOR, COLORREF backcolor, COLORREF forecolor, COLORREF selcolor, int zoom, int nLayer, BOOL blackandwhite)
{
	// Se c'e' un punto che vale (-1,-1), qualcuno ha fatto dei poltigli e
	// quindi non disegno nulla
	if( m_ptDefining[0].x == -1 && m_ptDefining[0].y == -1 ||
		m_ptDefining[1].x == -1 && m_ptDefining[1].y == -1)
		return;

	CPen *oldpen, pen( PS_SOLID, 0, m_bSelected && !bXNOR ? selcolor : forecolor);
	int oldROP = pdc->m_hAttribDC == NULL ?  R2_COPYPEN : pdc->GetROP2();
	CBrush *oldbrush, brush;

	if( bXNOR)
		pdc->SetROP2( R2_NOTXORPEN);
	else
		pdc->SetROP2( R2_COPYPEN);

	oldpen = pdc->SelectObject( &pen);

	if( !m_bFilled || bXNOR)
		brush.Attach( ::GetStockObject( NULL_BRUSH));
	else
		brush.CreateSolidBrush( m_bSelected ? selcolor : forecolor);
	oldbrush = pdc->SelectObject( &brush);

	// Il rettangolo lo facciamo con un poligono, cosi' non c'e' piu' il problema
	// del pixel in meno sul bordo
	CPoint pts[4];
	pts[0] = CPoint( m_ptDefining[0].x*zoom/100, m_ptDefining[0].y*zoom/100);
	pts[1] = CPoint( m_ptDefining[0].x*zoom/100, m_ptDefining[1].y*zoom/100);
	pts[2] = CPoint( m_ptDefining[1].x*zoom/100, m_ptDefining[1].y*zoom/100);
	pts[3] = CPoint( m_ptDefining[1].x*zoom/100, m_ptDefining[0].y*zoom/100);
	pdc->Polygon( pts, 4);

	pdc->SelectObject( oldbrush);
	pdc->SelectObject( oldpen);
	pen.DeleteObject();
	brush.DeleteObject();
	pdc->SetROP2( oldROP);

}

int CRectangle::GetNumDefPoints( void)
{
	return 2;
}

DWORD CRectangle::GetProp( void)
{
	return PROP_ROTATEFULL | PROP_SWAPFULL | PROP_UPDATEONZOOM;
}

BOOL CRectangle::GetAttribName( int index, CString& name, CString& values)
{
	if( index != 0)
		return FALSE;

	name = "Filled";
	values = "(Si No)";
	return TRUE;
}

BOOL CRectangle::GetAttrib( const char *name, CString& value)
{
	if( lstrcmp(name, "Filled"))
		return FALSE;

	value = m_bFilled ? "Si" : "No";
	return TRUE;
}

BOOL CRectangle::SetAttrib( const char *name, const char *value)
{
	if( lstrcmp( name, "Filled"))
		return FALSE;

	if( !lstrcmp( value, "Si"))
		m_bFilled = TRUE;
	else if( !lstrcmp( value, "No"))
		m_bFilled = FALSE;
	else
		return FALSE;

	/* Backup nuove impostazioni */
	_lastRect = *this;

	return TRUE;
}

void CRectangle::UpdateBoundingRgn( CDC *pdc)
{
	HRGN rg;
	rg = (HRGN)(m_Region.Detach());
	if( rg != NULL)
		::DeleteObject( rg);

	if( !m_bFilled)
	{
		int a;
		CRgn rgn;
		CPoint pt, ptold = CPoint( m_ptDefining[0].x, m_ptDefining[1].y);
		// rgn.CreateRectRgn( 0, 0, 1, 1);

		for( a=0; a<4; a++)
		{
			pt.x = (a==0 || a==3) ? m_ptDefining[0].x : m_ptDefining[1].x;
			pt.y = (a==0 || a==1) ? m_ptDefining[0].y : m_ptDefining[1].y;

			CPrimitive::RegioneLinea( pdc, rgn, ptold, pt, SELECTOR_SIZE);
			ptold = pt;
			if( m_Region.m_hObject == NULL)
			{
				m_Region.CreateRectRgn( 0, 0, 1, 1);
				m_Region.CopyRgn( &rgn);
			}
			else
				m_Region.CombineRgn( &m_Region, &rgn, RGN_OR);
		}
	}
	else
		m_Region.CreateRectRgn( m_ptDefining[0].x, m_ptDefining[0].y,
			m_ptDefining[1].x, m_ptDefining[1].y);
}

CPrimitive* CRectangle::Load( CString& linea)
{
	linea.TrimLeft();
	CString cmp = linea.Left(2);
	cmp.MakeUpper();

	if( cmp != "RP" && cmp != "RV")
		return NULL;

	CRectangle* prim = new CRectangle();
	prim->m_nLayer = 0;
	prim->m_bFilled = (cmp=="RP") ? TRUE : FALSE;	

	CString buf = linea.Right( linea.GetLength()-2);
	sscanf( LPCSTR(buf), "%d %d %d %d %d", &(prim->m_ptDefining[0].x),
		&(prim->m_ptDefining[0].y), &(prim->m_ptDefining[1].x),
		&(prim->m_ptDefining[1].y), &(prim->m_nLayer) );

	if( !prim->IsProper())
	{
		delete prim;
		return NULL;
	}
	
	prim->SaveState();
	return prim;
}

BOOL CRectangle::Save( CString& str, BOOL bSplit)
{
	str.Format( "%s %d %d %d %d",
		m_bFilled ? "RP" : "RV",
		m_ptDefining[0].x, m_ptDefining[0].y,
		m_ptDefining[1].x, m_ptDefining[1].y);

	if( m_nLayer == 0)
		str += "\r\n";
	else
	{
		CString l;
		l.Format( " %d\r\n", m_nLayer);
		str += l;
	}

	return TRUE;
}

void CRectangle::Serialize( CArchive &ar)
{
	if( ar.IsStoring())
	{
		ar <<	m_ptDefining[0] << m_ptDefining[1];
	}
	else
	{
		ar >>	m_ptDefining[0] >> m_ptDefining[1];
	}
}


///////////////////////////////////////////////////////////////////////////////////
// CMacro
///////////////////////////////////////////////////////////////////////////////////

CMacro::CMacro()
{
}

CMacro::CMacro( CPoint pt)
{
	m_ptDefining[0] = m_ptSaved[0] = pt;
}

CMacro::~CMacro()
{
	int a, n=m_Objects.GetSize();
	for( a=0; a<n; a++)
		delete (m_Objects[a]);
}

CPrimitive* CMacro::Clone()
{
	CMacro* p = new CMacro();
	*p = *(this);
	return p;
}

BOOL CMacro::operator= (CMacro& src)
{
	CPrimitive::operator=(src);

	m_sText = src.m_sText;

	// Svuota la precedente lista di oggetti...
	int a, n=m_Objects.GetSize();
	for( a=0; a<n; a++)
		delete (m_Objects[a]);
	m_Objects.RemoveAll();

	// ...e la riempie con la nuova (fa un hard-copy, non copia solo le reference!)
	n = src.m_Objects.GetSize();
	CPrimitive* m;
	for( a=0; a<n; a++)
	{
		m = src.m_Objects[a]->Clone();
		m_Objects.Add( m);
	}

	return TRUE;
}

PRIMARRAY* CMacro::GetComponentArray( void)
{
	return &m_Objects;
}

// Le macro si stampano dopo dei componenti normali
int CMacro::GetDrawPriority( void)
{
	return 4;
}

void CMacro::Draw( CDC *pdc, BOOL bXNOR, COLORREF backcolor, COLORREF forecolor, COLORREF selcolor, int zoom, int nLayer, BOOL blackandwhite)
{
	// Se nessun punto e' definito, non disegno nulla
	if( m_ptDefining[0].x == -1 && m_ptDefining[0].y == -1)
		return;

	/*
		Disegno degli oggetti componenti
		Chi chiama Draw() ha due possibilita`: passare -1 a nLayer (o non specificarlo), oppure passargli
		un numero da 0 a 15. Nel primo caso la Draw disegnera` tutti i layer (alla carlona), mentre
		nel secondo disegnera` soltanto quello specificato. Cio` vale ovviamente anche per le macro innestate.
	*/
	int ll, lp, lc, n=m_Objects.GetSize();
	CPrimitive *p;
	for( ll=0; ll<16; ll++)
	{
		// Stampo solo i layer visibili...
		if( pdc->IsPrinting())
		{
			if( !(((CMainFrame*)AfxGetMainWnd())->m_bLayerPrint[ll]))
				continue;
		}
		else
		{
			if( !(((CMainFrame*)AfxGetMainWnd())->m_bLayerShow[ll]))
				continue;
		}

		// ...solo quelli specificati...
		if( nLayer >= 0 && nLayer <= 15 && ll != nLayer)
			continue;

		// ...alla priorita` giusta
		for( lp=0; lp<=6; lp++)
		{
			for( lc=0; lc<n; lc++)
			{
				p = m_Objects[lc];

				// Le macro passano sempre (basta che siamo nella loro priorita`) perche` potrebbero
				// contenere componenti su qualsiasi layer
				if( ((p->GetProp() & PROP_ISMACRO) || p->GetLayer() == ll)
						&& p->GetDrawPriority() == lp)
				{
					/*
						Trova il colore con cui disegnare la primitiva
						(nero per la stampa)
					*/
					COLORREF clr = (blackandwhite || pdc->IsPrinting()) ? RGB(0,0,0) : ((CMainFrame *)AfxGetMainWnd())->m_layerColors[ll];

					// Disegna
					if( p->GetProp() & PROP_ISMACRO)
						((CMacro*)m_Objects[lc])->Draw( pdc, bXNOR, backcolor, clr, selcolor, zoom, ll);
					else
						m_Objects[lc]->Draw( pdc, bXNOR, backcolor, clr, selcolor, zoom);
				}
			}
		}
	}
}

void CMacro::UpdateBoundingRgn( CDC *pdc)
{
	/*
		La macro, composta da piu' oggetti, dovra' chiamare l'UpdateBoundingRgn()
		di tutti i suo componenti e combinare tutte le regioni. Potente, ma
		MOLTO lento... motivo in piu' per usare questa funzione con molto criterio.
	*/

	// Svuota la regione attualmente attiva
	HRGN rg;
	rg = (HRGN)(m_Region.Detach());
	if( rg != NULL)
		::DeleteObject( rg);

	// Poi inizia a fare le combinazioni
	int a, n=m_Objects.GetSize();
	CPrimitive *p;
	CRgn reg;
	for( a=0; a<n; a++)
	{
		p = m_Objects[a];
		p->UpdateBoundingRgn( pdc);

		if( reg.m_hObject == NULL)
		{
			reg.CreateRectRgn( 0, 0, 1, 1);
			reg.CopyRgn( p->GetRegion());
		}
		else
			reg.CombineRgn( &reg, p->GetRegion(), RGN_OR);
	}

	/*
		Problema: pensate a una macro fatta da 4 linee che formano un quadrato. La
		regione sara' composta dall'unione di 4 sottili striscie (le regioni delle
		linee), e tutta l'area interna non sara' considerata facente parte della macro.
		Questo non va bene, perche' l'utente si aspetta di selezionare una macro
		semplicemente cliccandovi dentro. Per ora useremo il bounding rectangle della
		macro come regione di riferimento: questo potrebbe portare a grossi errori, ma
		per ora non ho idee migliori.
	*/
	
	CRect rc;
	if( reg.m_hObject != NULL)
	{
		reg.GetRgnBox( &rc);
		m_Region.CreateRectRgn( rc.left, rc.top, rc.right, rc.bottom);
	}
}

int CMacro::GetNumDefPoints( void)
{
	return 1;
}

DWORD CMacro::GetProp( void)
{
	return PROP_ROTATEFULL | PROP_SWAPFULL | PROP_ISMACRO | PROP_UPDATEONZOOM;
}

BOOL CMacro::GetAttribName( int index, CString& name, CString& values)
{
	// Non vogliamo che venga mostrato il testo descrittivo delle macro,
	// che e` per uso interno!!
	return FALSE;
/*
	if( index != 0)
		return FALSE;

	name = "Text";
	values = "#STRING";
	return TRUE;*/
}

BOOL CMacro::GetAttrib( const char *name, CString& value)
{
	if( lstrcmp( name, "Text"))
		return FALSE;

	value = m_sText;
	return TRUE;
}

BOOL CMacro::SetAttrib( const char *name, const char *value)
{
	if( lstrcmp( name, "Text"))
		return FALSE;

	m_sText = value;
	return TRUE;
}

void CMacro::Rotate( CPoint pt, BOOL bCCV)
{
	CPrimitive::Rotate( pt, bCCV);

	int a, n=m_Objects.GetSize();
	for( a=0; a<n; a++)
		m_Objects[a]->Rotate( pt, bCCV);

	if( (!bCCV && !m_bSwapped) || (bCCV && m_bSwapped))
		m_nRotations++;
	else
		m_nRotations--;

	if( m_nRotations < 0)
		m_nRotations += 4;

	if( m_nRotations > 3)
		m_nRotations -= 4;
}

void CMacro::Swap( int xPos)
{
	CPrimitive::Swap( xPos);

	int a, n=m_Objects.GetSize();
	for( a=0; a<n; a++)
		m_Objects[a]->Swap( xPos);

	if( m_bSwapped)
		m_bSwapped = FALSE;
	else
		m_bSwapped = TRUE;
}

void CMacro::SetPoint( int nIndex, CPoint pt, BOOL bSignal)
{
	CPoint relpt = pt - m_ptDefining[0];
	
	int a, n=m_Objects.GetSize();
	for( a=0; a<n; a++)
		m_Objects[a]->MoveRel( relpt);

	CPrimitive::SetPoint( nIndex, pt);
}

void CMacro::MoveRel( CPoint ptrel)
{
	CPrimitive::MoveRel( ptrel);

	int a, n=m_Objects.GetSize();
	for( a=0; a<n; a++)
		m_Objects[a]->MoveRel( ptrel);
}

void CMacro::Select( BOOL bSel)
{
	int a,n=m_Objects.GetSize();
	for( a=0; a<n; a++)
		m_Objects[a]->Select( bSel);

	CPrimitive::Select( bSel);
}

void CMacro::SaveState( void)
{
	int a, n=m_Objects.GetSize();
	for( a=0; a<n; a++)
		m_Objects[a]->SaveState();

	CPrimitive::SaveState();
}

void CMacro::LoadState( void)
{
	int a, n=m_Objects.GetSize();
	for( a=0; a<n; a++)
		m_Objects[a]->LoadState();

	CPrimitive::LoadState();
}

CPrimitive* CMacro::Load( CString& linea)
{
	// Fase 1: parsing della linea
	linea.TrimLeft();
	CString cmp = linea.Left(2);
	cmp.MakeUpper();

	if( cmp != "MC")
		return NULL;
	
	int x, y, rot, swap;
	CString name;
	CMacro* prim = new CMacro();

	/* Niente layer per le macro */
	CString buf = linea.Right( linea.GetLength()-2);
	sscanf( LPCSTR(buf), "%d %d %d %d", &x, &y, &rot, &swap);
	name = ExtractString( buf, 4);
	
	// Fase 2: lettura del file di def della macro
	BOOL bStandard;
	if( !CMacro::ScanMacroName( name, *prim, NULL, &bStandard))
	{
		delete prim;
		return NULL;
	}

	prim->m_ptDefining[0].x = x;
	prim->m_ptDefining[0].y = y;
	prim->m_nRotations = rot;
	prim->m_bSwapped = swap;
	prim->SaveState();
	if( !prim->IsProper())
	{
		delete prim;
		return NULL;
	}

	// Se e' standard, vai di tilda
	if( bStandard)
		prim->m_sText = CString("~") + prim->m_sText;

	// Fase 3: aggiustamenti vari
	prim->Adjust();

	return prim;
}

void CMacro::Adjust( void)
{
	// Aggiustamento iniziale di posizione...
	int a, n=m_Objects.GetSize();
	for( a=0; a<n; a++)
		m_Objects[a]->MoveRel( CPoint( m_ptDefining[0].x - XMACRO_ORIG,
						m_ptDefining[0].y - YMACRO_ORIG));

	// ...aggiustamento di rotazione...
	int c, d=m_nRotations;
	for( c=0; c<d; c++)
		Rotate( m_ptDefining[0]);
	m_nRotations = d;


	// ...e aggiustamento di swap
	if( m_bSwapped)
	{
		Swap( m_ptDefining[0].x);
		m_bSwapped = TRUE;
	}
}

BOOL CMacro::Save( CString& str, BOOL bSplit)
{
	if( bSplit || m_sText == "")
	{
		int a, n=m_Objects.GetSize();
		CString temp;

		str.Empty();
		for( a=0; a<n; a++)
		{
			CString txt;
			m_Objects[a]->GetAttrib( "Text", txt);

			if( m_Objects[a]->GetProp() & PROP_ISMACRO && 
				txt.GetLength() > 0 && txt.GetAt(0) == '~')
					m_Objects[a]->Save( temp);
			else
					m_Objects[a]->Save( temp, bSplit);
			
			str += temp;
		}
	}
	else
	{
		CString macroname;
		if( m_sText[0] == '~')
			macroname = m_sText.Mid(1);
		else
			macroname = m_sText;

		str.Format( "MC %d %d %d %d %s\r\n",
			m_ptDefining[0].x, m_ptDefining[0].y, m_nRotations,
			m_bSwapped, LPCSTR(macroname));

		// NB: il layer non viene specificato per le macro, perche` non ha senso farlo
	}

	return TRUE;
}

void CMacro::Serialize( CArchive &ar)
{
	if( ar.IsStoring())
	{
		// TODO (future)
	}
	else
	{
		// TODO (future)
	}
}


/*
	ScanLibraryBuffer (statico)
	Individua i singoli blocchi-macro che compongono una libreria. Tali blocchi vengono
	semplicemente identificati e posti nello heap, senza alcun lavoro di decodifica.
	Per quest'ultima operazione e' necessario utilizzare ScanLibraryMacro.

	Inoltre ScanLibraryBuffer identifica anche il comando "subdir", identificato da una sintassi
	di questo tipo (all'inizio di una linea!):

	{Nome subdir}

	Il comando subdir deve essere posto fra le una macro e l'altra, e identifica la "sottodirectory"
	in cui le macro successive verranno messe. Questo e` utile per la rappresentazione visiva
	piu` ordinata delle librerie; inoltre definendo una directory speciale di nome "Hidden",
	le relative macro non verranno mostrate (utile per i componenti obsoleti). Specificando
	la directory vuota {} i prossimi componenti saranno ridisposti nella "root" della
	libreria.


	Parametri:
->	buffer: stringa che contiene il corpo della libreria (senza intestazione).
<-	LibBlocks: array di puntatori a CString. La funzione allochera' delle CString
	           nello heap e porra' in ognuna di esse un blocco-macro. Questa procedura
			   puo' sembrare un po' strana, ma e' un buon compromesso fra occupazione
			   di memoria e comodita'. Se l'array conteneva gia' dei puntatori, esso
			   verra' vuotato e le stringhe liberate.
<-	SubDirs: array di identificatori delle sottodirectory. Ogni identificatore contiene
			una stringa che rappresenta il nome della subdir, e un intero che
			rappresenta l'indice del primo componente di LibBlocks che deve essere
			mostrato in quella subdir. Notare che questi indici sono crescenti (non
			strettamente, nel senso che se ci sono due indici coincidenti ci sara`
			una subdir "vuota").

	Restituisce:
	TRUE in caso di successo, FALSE altrimenti.
*/

BOOL CMacro::ScanLibraryBuffer( CString& buffer, STRARRAY& lb, SUBDIRARRAY* sda)
{
	// Elimina eventuali stringhe preallocate
	int b, m=lb.GetSize();
	for( b=0; b<m; b++)
		delete (lb[b]);
	lb.RemoveAll();

	// Se il subdir array e` specificato, aggiunge una stringa vuota come
	// primo elemento, che identifica gli elementi "fuori dalle directory".
	if( sda != NULL)
	{
		SUBDIR sd;
		sd.index = 0;
		sd.name = "";
		sda->Add( sd);
	}

	// Suddivide i blocchi.
	int a=0, n=buffer.GetLength();
	CString riga;
	CString componente;
	while( a<n)
	{
		// Prossima riga
		int idx = buffer.Find( "\n", a);
		if( idx != -1)
		{
			riga = buffer.Mid( a, idx-a);
			riga.TrimRight();
			a=idx+1;
		}
		else
		{
			riga = buffer.Mid( a);
			riga.TrimRight();
			a=n;
		}

		// Controlla se e` uno header
		if( riga != "" && riga[0] == '[')
		{
			if( componente != "")
			{
				CString *str = new CString;
				*str = componente;
				lb.Add( str);
			}

			componente = riga + "\n";
		}
		// ...o una substringa...
		else if( riga != "" && riga[0] == '{')
		{
			if( componente != "")
			{
				CString *str = new CString;
				*str = componente;
				lb.Add( str);
				componente = "";
			}

			if( sda != NULL)
			{
				SUBDIR sd;
				sd.index = lb.GetSize();
				int idx2 = riga.Find( '}');
				if( idx2 != -1)
					sd.name = riga.Mid( 1, idx2-1);
				else
					sd.name = riga.Mid( 1);

				sda->Add( sd);
			}
		}
		// ...oppure lo accoda al componente attuale (se c'e`!!!)
		else
		{
			if( componente != "" && riga != "")
				componente += riga + "\n";
		}
	}

	// Salva componenti residui
	if( componente != "")
	{
		CString *str = new CString;
		*str = componente;
		lb.Add( str);
	}

	return TRUE;
}


/*
	ScanLibraryMacro (statico)
	Fa una cosa simile a ScanHeadedBuffer, ma sui blocchi-macro che compongono una
	libreria (che hanno un header leggermente diverso).

	Parametri:
->	buffer: stringa che contiene il blocco-macro (incluso l'header).
<-	dest: macro il cui array degli oggetti verra' inizializzato con i dati contenuti
	    nel buffer (la stringa di testo non viene invece modificata). Gli eventuali
		contenuti precedenti di tale array macro verranno distrutti. Se il 
		parametro e' NULL non verra' effettuato lo scan di tali oggetti (ma solo 
		dell'header).
<-	titleShort: puntatore a una stringa che ricevera' il titolo corto del blocco-macro.
	    Il parametro puo' essere NULL.
<-	titleLong: puntatore a una stringa che ricevera' il titolo lungo del blocco-macro.
	    Il parametro puo' essere NULL.
<-	bWarning: vedi ScanBuffer.

	Restituisce:
	TRUE in caso di successo, FALSE altrimenti (sostanzialmente per header invalidi).
*/

BOOL CMacro::ScanLibraryMacro( CString& buffer, CMacro* dest, CString* titleShort,
	CString* titleLong, BOOL* bWarning)
{
	if( bWarning != NULL)
		*bWarning = FALSE;

	// Il primo carattere diverso da spazi DEVE essere '['
	CString str1 = buffer;
	str1.TrimLeft();
	if( str1.GetLength() < 1 || str1[0] != '[')
		return FALSE;

	// Estrae l'header
	CString str2 = str1.Mid(1);
	str2.TrimLeft();
	int idx = str2.Find(']');
	if( idx == -1)
		return FALSE;
	CString header = str2.Left(idx);
	
	// Estrae il titolo breve...
	int tb = header.Find(' ');
	if( idx == -1)
		return FALSE;
	if( titleShort != NULL)
		*titleShort = header.Left(tb);

	// ...poi quello lungo
	CString str3 = header.Mid(tb);
	str3.TrimLeft();
	if( titleLong != NULL)
		*titleLong = str3;

	// Poi (se richiesto) scanna anche il corpo del blocco-macro
	if( dest == NULL)
		return TRUE;
	
	// Verifica che non sia vuoto (questo comunque non e' un errore)
	int start = buffer.Find(']');
	if( start >= buffer.GetLength()-1)
		return TRUE;

	CString body = buffer.Mid(start+1);
	return CPrimitive::ScanBuffer( body, *(dest->GetComponentArray()), bWarning);
}



/*
	ScanMacroName (statico)
	Crea una macro e/o legge il suo nome lungo, dato il nome breve (nella forma
	passata al comando MC). Ci sono 2 casi:
	1) Nome del tipo "libreria.macro". In tal caso la funzione legge il blocco-macro
	   nella libreria corrispondente.
	2) Nome del tipo "macro". In tal caso la funzione esegue lo scan della libreria
	   standard alla ricerca del blocco-macro dotato di tale nome, e solo se non lo
	   trova ripiega sulle macro "sfuse".

	Parametri:
->	macroname: nome breve della macro, nella forma usata nel comando MC.
<-	dest: oggetto che conterra' la macro caricata. Gli eventuali
		contenuti precedenti di tale macro verranno distrutti. Se il 
		parametro e' NULL non verra' effettuato lo scan del corpo della macro
		(ma solo dell'header).
<-	title: puntatore alla stringa che ricevera' il nome lungo della macro. Il parametro
	       puo' essere NULL.
<-	bWarning: vedi ScanBuffer.

	Restituisce:
	TRUE in caso di successo, FALSE altrimenti (file non trovato, header invalido,
	errore in caricamento, ecc).
*/

BOOL CMacro::ScanMacroName( CString& mname, CMacro& dest, BOOL *bWarning,
	BOOL *bStandard)
{
	if( bWarning != NULL)
		*bWarning = FALSE;

	if( bStandard != NULL)
	{
		if( mname.Find(".") != -1 && mname[0] == '~')
			*bStandard = TRUE;
		else
			*bStandard = FALSE;
	}

	// Innanzitutto verifica se il componente specificato in mname e' gia' stato
	// caricato in passato (nel caso di macro standard la tilda puo essere specificata
	// o meno, e in ogni caso nella macro restituita ci sara').

	CString nametilded = (mname[0]=='~') ? mname : CString("~")+mname;
	CMacro* ptmacro;
	if( _MacroCache.Lookup( mname, (CObject*&)ptmacro))
	{
		// bingo!
		dest = *ptmacro;

		// Controlla se e' una macro standard "implicita"
		if( mname[0] == '~' || _MacroCache.Lookup( nametilded, (CObject*&)ptmacro))
			dest.SetAttrib( "Text", nametilded);
		else
			dest.SetAttrib( "Text", mname);

		return TRUE;
	}

	// Verifica se il nome e' nella forma "lib.macro" o nella forma "macro"
	int idx = mname.Find('.');
	if( idx != -1)
	{
		CString LibName = mname.Left(idx);
		LibName.TrimLeft(); LibName.TrimRight();
		CString LibFile( _BaseDir + CString("\\" LIBS_DIR "\\") + LibName + 
							CString("." LIBS_EXT));

		DOC_TYPE t;
		STRARRAY libcomps;
		if( !CPrimitive::ScanFile( LibFile, NULL, &libcomps, NULL, NULL, &t) ||
				t != DOC_LIBRARY)
			return FALSE;

		// Cerca fra le macro contenute nella libreria
		CString mcname = mname.Mid(idx+1);
		mcname.TrimLeft(); mcname.TrimRight();
		CString titlebrief;
		int a, n=libcomps.GetSize();
		for( a=0; a<n; a++)
		{
			if( CMacro::ScanLibraryMacro( *(libcomps[a]), NULL, &titlebrief,
				NULL, NULL) && titlebrief == mcname)
			{
				if( CMacro::ScanLibraryMacro( *libcomps[a], &dest, NULL,
					NULL, bWarning))
				{
					break;
				}
			}
		}

		// Elimina la mem allocata
		for( int b=0; b<n; b++)
			delete (libcomps[b]);

		// Ha trovato una macro valida?
		if( a==n)
			return FALSE;
		else
		{
			// la memorizza (in un'istanza a parte) e ritorna TRUE
			dest.SetAttrib( "Text", mname);
			CPrimitive *p = dest.Clone();
			_MacroCache.SetAt( mname, p);
			return TRUE;
		}
	}
	else
	{
		CString macroname;
		
		// Pulisce la tilda dai simboli standard
		// (serve solo per riconoscerli in fase di salvataggio)
		if( mname[0] == '~')
			macroname = mname.Mid(1);
		else
			macroname = mname;

		// Cerca prima nella libreria standard...
		STRARRAY stdlib;
		BOOL bResult;
		LPSTR lpRes;
		HANDLE hResInfo, hRes;
		hResInfo = ::FindResource( NULL, MAKEINTRESOURCE(IDR_STDLIB), "STDLIBS");
		if( hResInfo != NULL)
		{
    		hRes = ::LoadResource( NULL, (HRSRC)hResInfo);
    		if (hRes != NULL)
			{
    			lpRes = (LPSTR)(::LockResource( hRes));
    			if( lpRes != NULL)
    			{
					CString str( lpRes);
					bResult = CPrimitive::ScanHeadedBuffer( str, NULL, &stdlib,
						NULL, NULL, NULL);
        			::UnlockResource( hRes);
    			}
    			::FreeResource( hRes);
			}
		}

		if( bResult)
		{
			int a, n=stdlib.GetSize();
			CString Short;
			for( a=0; a<n; a++)
			{
				if( CMacro::ScanLibraryMacro( *(stdlib[a]), NULL, &Short,
						NULL, NULL) && Short == macroname)
				{
					if( CMacro::ScanLibraryMacro( *(stdlib[a]), &dest, NULL,
						NULL, bWarning))
					{
						break;
					}
				}
			}

			// Elimina la mem allocata
			for( int b=0; b<n; b++)
				delete (stdlib[b]);

			// Ha trovato una macro valida?
			if( a<n)
			{
				// la memorizza (in due istanze a parte, una con tilda e una senza)
				// e ritorna TRUE
				dest.SetAttrib( "Text", nametilded);
				CPrimitive *p = dest.Clone(), *r = dest.Clone();
				_MacroCache.SetAt( mname, p);
				_MacroCache.SetAt( nametilded, r);
				return TRUE;
			}
		}
		
		// altrimenti cerca fra le macro sfuse
		CString MacroFile( _BaseDir + CString("\\" MACROS_DIR "\\") + 
					macroname + CString("." MACROS_EXT));
		if( CPrimitive::ScanFile( MacroFile, dest.GetComponentArray(), NULL,
				NULL, NULL, NULL))
		{
			// macro valida? La memorizza (in un'istanza a parte) e ritorna TRUE
			// (verifica che il nome non sia usato anche da una macro STD)
			CPrimitive *q;
			if( !_MacroCache.Lookup( nametilded, (CObject*&)q))
			{
				dest.SetAttrib( "Text", mname);
				CPrimitive *p = dest.Clone();
				_MacroCache.SetAt( mname, p);
			}
			return TRUE;
		}
		else
			return FALSE;
	}
}


///////////////////////////////////////////////////////////////////////////////////
// CPCBLine
///////////////////////////////////////////////////////////////////////////////////

CPCBLine::CPCBLine()
{
	m_thickness = 2;
}

CPCBLine::CPCBLine( CPoint start, int thickness, CPoint stop, BOOL bConst)
{
	// Pesca ultimi settaggi dal pool
	*this = _lastVia;

	m_ptDefining[0] = m_ptSaved[0] = start;
	m_ptDefining[1] = m_ptSaved[1] = stop;
	m_bConst = bConst;
}

CPCBLine::~CPCBLine()
{
}

CPrimitive* CPCBLine::Clone()
{
	CPCBLine* p = new CPCBLine();
	*p = *(this);
	p->m_thickness = m_thickness;
	return p;
}

void CPCBLine::Draw( CDC *pdc, BOOL bXNOR, COLORREF backcolor, COLORREF forecolor, COLORREF selcolor, int zoom, int nLayer, BOOL blackandwhite)
{
	// Se c'e' un punto che vale (-1,-1), qualcuno ha fatto dei poltigli e
	// quindi non disegno nulla
	if( m_ptDefining[0].x == -1 && m_ptDefining[0].y == -1 ||
		m_ptDefining[1].x == -1 && m_ptDefining[1].y == -1)
		return;

	CPen *oldpen, pen( PS_GEOMETRIC, m_thickness*zoom/100, m_bSelected && !bXNOR ? selcolor : forecolor);
	int oldROP = pdc->m_hAttribDC == NULL ?  R2_COPYPEN : pdc->GetROP2();

	if( bXNOR)
		pdc->SetROP2( R2_NOTXORPEN);
	else
		pdc->SetROP2( R2_COPYPEN);

	oldpen = pdc->SelectObject( &pen);

	pdc->MoveTo( m_ptDefining[0].x*zoom/100, m_ptDefining[0].y*zoom/100);
	pdc->LineTo( m_ptDefining[1].x*zoom/100, m_ptDefining[1].y*zoom/100);
	pdc->SetPixel( m_ptDefining[1].x*zoom/100, m_ptDefining[1].y*zoom/100, forecolor);

	pdc->SelectObject( oldpen);
	pen.DeleteObject();
	pdc->SetROP2( oldROP);
}

void CPCBLine::UpdateBoundingRgn( CDC *pdc)
{
	// Guarda se e` piu` larga la linea o il selettore, e passa la dimensione fisica
	// del piu` largo
	CSize l( m_thickness, m_thickness);
	pdc->LPtoDP( &l);
	int maxlargh = (l.cx > l.cy) ? l.cx : l.cy;

	CPrimitive::RegioneLinea( pdc, m_Region, m_ptDefining[0], m_ptDefining[1],
		SELECTOR_SIZE > maxlargh ? SELECTOR_SIZE : maxlargh);
}

int CPCBLine::GetNumDefPoints( void)
{
	return 2;
}

DWORD CPCBLine::GetProp( void)
{
	return PROP_ROTATEFULL | PROP_SWAPFULL | PROP_UPDATEONZOOM | PROP_SERIALCREATE;
}

BOOL CPCBLine::GetAttribName( int index, CString& name, CString& values)
{
	if( index != 0)
		return FALSE;

	name = "Thickness";
	values = "#INT 1 100";
	return TRUE;
}

BOOL CPCBLine::GetAttrib( const char *name, CString& value)
{
	if( lstrcmp(name, "Thickness"))
		return FALSE;

	value.Format( "%d", m_thickness);
	return TRUE;
}

BOOL CPCBLine::SetAttrib( const char *name, const char *value)
{
	if( lstrcmp( name, "Thickness"))
		return FALSE;

	int n = atoi( value);
	if( n<1 || n>100)
		return FALSE;
	
	m_thickness = n;

	/* Backup nuove impostazioni */
	_lastVia = *this;

	return TRUE;
}


CPrimitive* CPCBLine::Load( CString& linea)
{
	linea.TrimLeft();
	CString cmp = linea.Left(2);
	cmp.MakeUpper();

	if( cmp != "PL")
		return NULL;
	
	CPCBLine* prim = new CPCBLine();
	prim->m_nLayer = 0;
	CString buf = linea.Right( linea.GetLength()-2);
	sscanf( LPCSTR(buf), "%d %d %d %d %d %d", &(prim->m_ptDefining[0].x),
		&(prim->m_ptDefining[0].y), &(prim->m_ptDefining[1].x),
		&(prim->m_ptDefining[1].y), &(prim->m_thickness), &(prim->m_nLayer));

	if( !prim->IsProper())
	{
		delete prim;
		return NULL;
	}
	
	prim->SaveState();
	return prim;
}

BOOL CPCBLine::Save( CString& str, BOOL bSplit)
{
	str.Format( "PL %d %d %d %d %d", m_ptDefining[0].x, m_ptDefining[0].y,
		m_ptDefining[1].x, m_ptDefining[1].y, m_thickness);

	if( m_nLayer == 0)
		str += "\r\n";
	else
	{
		CString l;
		l.Format( " %d\r\n", m_nLayer);
		str += l;
	}

	return TRUE;
}

BOOL CPCBLine::IsProper()
{
	// TODO: funziona?
	if( !CPrimitive::IsProper())
		return FALSE;

	if( m_thickness < 1)
		return FALSE;

	return TRUE;
}

void CPCBLine::Serialize( CArchive &ar)
{
	if( ar.IsStoring())
	{
		ar <<	m_ptDefining[0] << m_ptDefining[1] << m_thickness;
	}
	else
	{
		ar >>	m_ptDefining[0] >> m_ptDefining[1] >> m_thickness;
	}
}


///////////////////////////////////////////////////////////////////////////////////
// CPCBPad
///////////////////////////////////////////////////////////////////////////////////

CPCBPad::CPCBPad()
{
	m_sizex = m_sizey = 9;
	m_intdiam = 3;
	m_style = PAD_ROUND;
}

CPCBPad::CPCBPad( CPoint start, int sizex, int sizey, int intdiam, int style, BOOL bConst)
{
	// Pesca ultimi settaggi dal pool
	*this = _lastPad;

	m_ptDefining[0] = m_ptSaved[0] = start;
/*	m_sizex = sizex;
	m_sizey = sizey;
	m_intdiam = intdiam;
	m_style = style;*/
	m_bConst = bConst;
}

CPCBPad::~CPCBPad()
{
}

CPrimitive* CPCBPad::Clone()
{
	CPCBPad* p = new CPCBPad();
	*p = *(this);
	p->m_sizex = m_sizex;
	p->m_sizey = m_sizey;
	p->m_intdiam = m_intdiam;
	p->m_style = m_style;
	return p;
}

// I componenti PCB si disegnano dopo gli altri
int CPCBPad::GetDrawPriority( void)
{
	return 5;
}

void CPCBPad::Draw( CDC *pdc, BOOL bXNOR, COLORREF backcolor, COLORREF forecolor, COLORREF selcolor, int zoom, int nLayer, BOOL blackandwhite)
{
	// Se c'e' un punto che vale (-1,-1), qualcuno ha fatto dei poltigli e
	// quindi non disegno nulla
	if( m_ptDefining[0].x == -1 && m_ptDefining[0].y == -1)
		return;

	int oldROP = pdc->m_hAttribDC == NULL ?  R2_COPYPEN : pdc->GetROP2();
	if( bXNOR)
		pdc->SetROP2( R2_NOTXORPEN);
	else
		pdc->SetROP2( R2_COPYPEN);

	CPen *oldpen, pen( PS_SOLID, 0, m_bSelected && !bXNOR ? selcolor : forecolor);
	oldpen = pdc->SelectObject( &pen);

	CBrush *oldbrush, forebrush, backbrush;
	forebrush.CreateSolidBrush( m_bSelected ? selcolor : forecolor);
	backbrush.CreateSolidBrush( backcolor);

	/*
		Ci sono tre stili per le piazzole: rotonde, rettangolari e rettangolari con
		spigoli arrotondati. Per semplicita` prima disegnamo l'outline senza buco, poi
		disegnamo il buco del colore dello sfondo. Sara` compito del motore di rendering,
		aiutato dalla funzione GetDrawPriority(), disegnare la primitiva dopo delle altre
		in modo che si veda il buco.
	*/

	// Disegna il contorno
	oldbrush = pdc->SelectObject( &forebrush);
	if( m_style == PAD_ROUND)
		pdc->Ellipse(	(m_ptDefining[0].x - m_sizex/2)*zoom/100,
						(m_ptDefining[0].y - m_sizey/2)*zoom/100,
						(m_ptDefining[0].x + m_sizex/2)*zoom/100,
						(m_ptDefining[0].y + m_sizey/2)*zoom/100);
	else if( m_style == PAD_SQUARE)
		pdc->Rectangle(	(m_ptDefining[0].x - m_sizex/2)*zoom/100,
						(m_ptDefining[0].y - m_sizey/2)*zoom/100,
						(m_ptDefining[0].x + m_sizex/2)*zoom/100,
						(m_ptDefining[0].y + m_sizey/2)*zoom/100);
	else
		pdc->RoundRect( (m_ptDefining[0].x - m_sizex/2)*zoom/100,
						(m_ptDefining[0].y - m_sizey/2)*zoom/100,
						(m_ptDefining[0].x + m_sizex/2)*zoom/100,
						(m_ptDefining[0].y + m_sizey/2)*zoom/100,
						m_sizex*zoom/200, m_sizey*zoom/200);

	// Disegna il buco
	pdc->SelectObject( &backbrush);
	pdc->Ellipse(	(m_ptDefining[0].x - m_intdiam/2)*zoom/100,
					(m_ptDefining[0].y - m_intdiam/2)*zoom/100,
					(m_ptDefining[0].x + m_intdiam/2)*zoom/100,
					(m_ptDefining[0].y + m_intdiam/2)*zoom/100);

	pdc->SelectObject( oldbrush);
	pdc->SelectObject( oldpen);
	pen.DeleteObject();
	forebrush.DeleteObject();
	backbrush.DeleteObject();
	pdc->SetROP2( oldROP);
}

void CPCBPad::UpdateBoundingRgn( CDC *pdc)
{
	// Elimina la vecchia regione associata
	HRGN rg;
	rg = (HRGN)(m_Region.Detach());
	if( rg != NULL)
		::DeleteObject( rg);

	// Crea la regione del contorno
	if( m_style == PAD_ROUND)
		m_Region.CreateEllipticRgn( m_ptDefining[0].x - m_sizex/2 - 1,
									m_ptDefining[0].y - m_sizey/2 - 1,
									m_ptDefining[0].x + m_sizex/2 + 1,
									m_ptDefining[0].y + m_sizey/2 + 1);
	else
		m_Region.CreateRectRgn(		m_ptDefining[0].x - m_sizex/2 - 1,
									m_ptDefining[0].y - m_sizey/2 - 1,
									m_ptDefining[0].x + m_sizex/2 + 1,
									m_ptDefining[0].y + m_sizey/2 + 1);
}

int CPCBPad::GetNumDefPoints( void)
{
	return 1;
}

DWORD CPCBPad::GetProp( void)
{
	return PROP_ROTATEFULL | PROP_SWAPFULL | PROP_UPDATEONZOOM;
}

BOOL CPCBPad::GetAttribName( int index, CString& name, CString& values)
{
	switch( index)
	{
		case 0:
			name = "SizeX";
			values = "#INT 2 100";
			return TRUE;

		case 1:
			name = "SizeY";
			values = "#INT 2 100";
			return TRUE;

		case 2:
			name = "IntDiam";
			values = "#INT 2 100";
			return TRUE;

		case 3:
			name = "Style";
			values = "(Round Square SquareRounded)";
			return TRUE;

		default:
			return FALSE;
	}
}

BOOL CPCBPad::GetAttrib( const char *name, CString& value)
{
	if( !lstrcmp(name, "SizeX"))
		value.Format( "%d", m_sizex);
	else if( !lstrcmp(name, "SizeY"))
		value.Format( "%d", m_sizey);
	else if( !lstrcmp(name, "IntDiam"))
		value.Format( "%d", m_intdiam);
	else if( !lstrcmp(name, "Style"))
		value.Format( "%s", m_style==PAD_ROUND ? "Round" :
							m_style==PAD_SQUARE ? "Square" : "SquareRounded");
	else
		return FALSE;

	return TRUE;
}

BOOL CPCBPad::SetAttrib( const char *name, const char *value)
{
	int n = atoi( value);

	if( !lstrcmp(name, "SizeX"))
	{
		if( n >= 2 && n <= 100)
			m_sizex = n;
		else
			return FALSE;
	}
	else if( !lstrcmp(name, "SizeY"))
	{
		if( n >= 2 && n <= 100)
			m_sizey = n;
		else
			return FALSE;
	}
	else if( !lstrcmp(name, "IntDiam"))
	{
		if( n >= 2 && n <= 100)
			m_intdiam = n;
		else
			return FALSE;
	}
	else if( !lstrcmp(name, "Style"))
	{
		if( !lstrcmp( value, "Round"))
			m_style = PAD_ROUND;
		else if( !lstrcmp( value, "Square"))
			m_style = PAD_SQUARE;
		else if( !lstrcmp( value, "SquareRounded"))
			m_style = PAD_SQUARE_ROUNDED;
		else
			return FALSE;
	}

	/* Backup nuove impostazioni */
	_lastPad = *this;

	return TRUE;
}

void CPCBPad::Rotate( CPoint pt, BOOL bCCV)
{
	CPrimitive::Rotate( pt, bCCV);

	// Ruoto la piazzola semplicemente scambiando le sue dimensioni x-y
	int tmp = m_sizex;
	m_sizex = m_sizey;
	m_sizey = tmp;
}

CPrimitive* CPCBPad::Load( CString& linea)
{
	linea.TrimLeft();
	CString cmp = linea.Left(2);
	cmp.MakeUpper();

	if( cmp != "PA")
		return NULL;
	
	CPCBPad* prim = new CPCBPad();
	prim->m_nLayer = 0;
	CString buf = linea.Right( linea.GetLength()-2);
	sscanf( LPCSTR(buf), "%d %d %d %d %d %d %d", &(prim->m_ptDefining[0].x),
		&(prim->m_ptDefining[0].y), &(prim->m_sizex), &(prim->m_sizey),
		&(prim->m_intdiam), &(prim->m_style), &(prim->m_nLayer));

	if( !prim->IsProper())
	{
		delete prim;
		return NULL;
	}
	
	prim->SaveState();
	return prim;
}

BOOL CPCBPad::Save( CString& str, BOOL bSplit)
{
	str.Format( "PA %d %d %d %d %d %d", m_ptDefining[0].x, m_ptDefining[0].y,
		m_sizex, m_sizey, m_intdiam, m_style);

	if( m_nLayer == 0)
		str += "\r\n";
	else
	{
		CString l;
		l.Format( " %d\r\n", m_nLayer);
		str += l;
	}

	return TRUE;
}

BOOL CPCBPad::IsProper()
{
	if( !CPrimitive::IsProper())
		return FALSE;

	if( m_sizex < 1 || m_sizey < 1 || m_intdiam < 1 ||
		m_sizex <= m_intdiam || m_sizey <= m_intdiam)
		return FALSE;

	return TRUE;
}

void CPCBPad::Serialize( CArchive &ar)
{
	if( ar.IsStoring())
	{
		ar <<	m_ptDefining[0] << m_sizex << m_sizey << m_intdiam << m_style;
	}
	else
	{
		ar >>	m_ptDefining[0] >> m_sizex >> m_sizey >> m_intdiam >> m_style;
	}
}
